@@ -28,8 +28,9 @@
 package org.apache.http.impl.nio;
 
 import java.io.IOException;
-import java.net.InetSocketAddress;
-import java.net.SocketAddress;
+import java.nio.channels.SelectionKey;
+import java.nio.charset.CharsetDecoder;
+import java.nio.charset.CharsetEncoder;
 
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpEntityEnclosingRequest;
@@ -37,11 +38,21 @@ import org.apache.http.HttpException;
 import org.apache.http.HttpRequest;
 import org.apache.http.HttpRequestFactory;
 import org.apache.http.HttpResponse;
+import org.apache.http.annotation.NotThreadSafe;
+import org.apache.http.config.MessageConstraints;
+import org.apache.http.entity.ContentLengthStrategy;
+import org.apache.http.impl.entity.DisallowIdentityContentLengthStrategy;
+import org.apache.http.impl.entity.LaxContentLengthStrategy;
+import org.apache.http.impl.entity.StrictContentLengthStrategy;
 import org.apache.http.impl.nio.codecs.DefaultHttpRequestParser;
+import org.apache.http.impl.nio.codecs.DefaultHttpRequestParserFactory;
 import org.apache.http.impl.nio.codecs.DefaultHttpResponseWriter;
+import org.apache.http.impl.nio.codecs.DefaultHttpResponseWriterFactory;
 import org.apache.http.nio.NHttpMessageParser;
+import org.apache.http.nio.NHttpMessageParserFactory;
 import org.apache.http.nio.NHttpMessageWriter;
-import org.apache.http.nio.NHttpServerConnection;
+import org.apache.http.nio.NHttpMessageWriterFactory;
+import org.apache.http.nio.NHttpServerEventHandler;
 import org.apache.http.nio.NHttpServerIOTarget;
 import org.apache.http.nio.NHttpServiceHandler;
 import org.apache.http.nio.reactor.EventMask;
@@ -49,22 +60,18 @@ import org.apache.http.nio.reactor.IOSession;
 import org.apache.http.nio.reactor.SessionInputBuffer;
 import org.apache.http.nio.reactor.SessionOutputBuffer;
 import org.apache.http.nio.util.ByteBufferAllocator;
+import org.apache.http.params.HttpParamConfig;
 import org.apache.http.params.HttpParams;
+import org.apache.http.util.Args;
 
 /**
- * Default implementation of the {@link NHttpServerConnection} interface.
- * <p>
- * The following parameters can be used to customize the behavior of this
- * class:
- * <ul>
- *  <li>{@link org.apache.http.params.CoreProtocolPNames#HTTP_ELEMENT_CHARSET}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#SOCKET_BUFFER_SIZE}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#MAX_HEADER_COUNT}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#MAX_LINE_LENGTH}</li>
- * </ul>
+ * Default implementation of the {@link org.apache.http.nio.NHttpServerConnection}
+ * interface.
  *
  * @since 4.0
  */
+@SuppressWarnings("deprecation")
+@NotThreadSafe
 public class DefaultNHttpServerConnection
     extends NHttpConnectionBase implements NHttpServerIOTarget {
 
@@ -78,20 +85,102 @@ public class DefaultNHttpServerConnection
      * @param requestFactory HTTP request factory.
      * @param allocator byte buffer allocator.
      * @param params HTTP parameters.
+     *
+     * @deprecated (4.3) use {@link DefaultNHttpServerConnection#DefaultNHttpServerConnection(
+     *   IOSession, int, int, ByteBufferAllocator, CharsetDecoder, CharsetEncoder,
+     *   MessageConstraints, ContentLengthStrategy, ContentLengthStrategy,
+     *   NHttpMessageParserFactory, NHttpMessageWriterFactory)}
      */
+    @Deprecated
     public DefaultNHttpServerConnection(
             final IOSession session,
             final HttpRequestFactory requestFactory,
             final ByteBufferAllocator allocator,
             final HttpParams params) {
         super(session, allocator, params);
-        if (requestFactory == null) {
-            throw new IllegalArgumentException("Request factory may not be null");
-        }
+        Args.notNull(requestFactory, "Request factory");
         this.requestParser = createRequestParser(this.inbuf, requestFactory, params);
         this.responseWriter = createResponseWriter(this.outbuf, params);
     }
 
+    /**
+     * Creates new instance DefaultNHttpServerConnection given the underlying I/O session.
+     *
+     * @param session the underlying I/O session.
+     * @param buffersize buffer size. Must be a positive number.
+     * @param fragmentSizeHint fragment size hint.
+     * @param allocator memory allocator.
+     *   If <code>null</code> {@link org.apache.http.nio.util.HeapByteBufferAllocator#INSTANCE}
+     *   will be used.
+     * @param chardecoder decoder to be used for decoding HTTP protocol elements.
+     *   If <code>null</code> simple type cast will be used for byte to char conversion.
+     * @param charencoder encoder to be used for encoding HTTP protocol elements.
+     *   If <code>null</code> simple type cast will be used for char to byte conversion.
+     * @param constraints Message constraints. If <code>null</code>
+     *   {@link MessageConstraints#DEFAULT} will be used.
+     * @param incomingContentStrategy incoming content length strategy. If <code>null</code>
+     *   {@link DisallowIdentityContentLengthStrategy#INSTANCE} will be used.
+     * @param outgoingContentStrategy outgoing content length strategy. If <code>null</code>
+     *   {@link StrictContentLengthStrategy#INSTANCE} will be used.
+     * @param requestParserFactory request parser factory. If <code>null</code>
+     *   {@link DefaultHttpRequestParserFactory#INSTANCE} will be used.
+     * @param responseWriterFactory response writer factory. If <code>null</code>
+     *   {@link DefaultHttpResponseWriterFactory#INSTANCE} will be used.
+     *
+     * @since 4.3
+     */
+    public DefaultNHttpServerConnection(
+            final IOSession session,
+            final int buffersize,
+            final int fragmentSizeHint,
+            final ByteBufferAllocator allocator,
+            final CharsetDecoder chardecoder,
+            final CharsetEncoder charencoder,
+            final MessageConstraints constraints,
+            final ContentLengthStrategy incomingContentStrategy,
+            final ContentLengthStrategy outgoingContentStrategy,
+            final NHttpMessageParserFactory<HttpRequest> requestParserFactory,
+            final NHttpMessageWriterFactory<HttpResponse> responseWriterFactory) {
+        super(session, buffersize, fragmentSizeHint, allocator, chardecoder, charencoder,
+                incomingContentStrategy != null ? incomingContentStrategy :
+                    DisallowIdentityContentLengthStrategy.INSTANCE,
+                outgoingContentStrategy != null ? outgoingContentStrategy :
+                    StrictContentLengthStrategy.INSTANCE);
+        this.requestParser = (requestParserFactory != null ? requestParserFactory :
+            DefaultHttpRequestParserFactory.INSTANCE).create(this.inbuf, constraints);
+        this.responseWriter = (responseWriterFactory != null ? responseWriterFactory :
+            DefaultHttpResponseWriterFactory.INSTANCE).create(this.outbuf);
+    }
+
+    /**
+     * @since 4.3
+     */
+    public DefaultNHttpServerConnection(
+            final IOSession session,
+            final int buffersize,
+            final CharsetDecoder chardecoder,
+            final CharsetEncoder charencoder,
+            final MessageConstraints constraints) {
+        this(session, buffersize, buffersize, null, chardecoder, charencoder, constraints,
+                null, null, null, null);
+    }
+
+    /**
+     * @since 4.3
+     */
+    public DefaultNHttpServerConnection(final IOSession session, final int buffersize) {
+        this(session, buffersize, buffersize, null, null, null, null, null, null, null, null);
+    }
+
+    /**
+     * @deprecated (4.3) use constructor.
+     */
+    @Override
+    @Deprecated
+    protected ContentLengthStrategy createIncomingContentStrategy() {
+        return new DisallowIdentityContentLengthStrategy(new LaxContentLengthStrategy(0));
+    }
+
     /**
      * Creates an instance of {@link NHttpMessageParser} to be used
      * by this connection for parsing incoming {@link HttpRequest} messages.
@@ -100,13 +189,16 @@ public class DefaultNHttpServerConnection
      * a different implementation of the {@link NHttpMessageParser} interface.
      *
      * @return HTTP response parser.
+     *
+     * @deprecated (4.3) use constructor.
      */
+    @Deprecated
     protected NHttpMessageParser<HttpRequest> createRequestParser(
             final SessionInputBuffer buffer,
             final HttpRequestFactory requestFactory,
             final HttpParams params) {
-        // override in derived class to specify a line parser
-        return new DefaultHttpRequestParser(buffer, null, requestFactory, params);
+        final MessageConstraints constraints = HttpParamConfig.getMessageConstraints(params);
+        return new DefaultHttpRequestParser(buffer, null, requestFactory, constraints);
     }
 
     /**
@@ -118,12 +210,27 @@ public class DefaultNHttpServerConnection
      * a different implementation of the {@link NHttpMessageWriter} interface.
      *
      * @return HTTP response parser.
+     *
+     * @deprecated (4.3) use constructor.
      */
+    @Deprecated
     protected NHttpMessageWriter<HttpResponse> createResponseWriter(
             final SessionOutputBuffer buffer,
             final HttpParams params) {
         // override in derived class to specify a line formatter
-        return new DefaultHttpResponseWriter(buffer, null, params);
+        return new DefaultHttpResponseWriter(buffer, null);
+    }
+
+    /**
+     * @since 4.2
+     */
+    protected void onRequestReceived(final HttpRequest request) {
+    }
+
+    /**
+     * @since 4.2
+     */
+    protected void onResponseSubmitted(final HttpResponse response) {
     }
 
     public void resetInput() {
@@ -138,7 +245,7 @@ public class DefaultNHttpServerConnection
         this.responseWriter.reset();
     }
 
-    public void consumeInput(final NHttpServiceHandler handler) {
+    public void consumeInput(final NHttpServerEventHandler handler) {
         if (this.status != ACTIVE) {
             this.session.clearEvent(EventMask.READ);
             return;
@@ -156,14 +263,12 @@ public class DefaultNHttpServerConnection
                 if (this.request != null) {
                     if (this.request instanceof HttpEntityEnclosingRequest) {
                         // Receive incoming entity
-                        HttpEntity entity = prepareDecoder(this.request);
+                        final HttpEntity entity = prepareDecoder(this.request);
                         ((HttpEntityEnclosingRequest)this.request).setEntity(entity);
                     }
                     this.connMetrics.incrementRequestCount();
+                    onRequestReceived(this.request);
                     handler.requestReceived(this);
-                    if (this.session.isClosed()) {
-                        return;
-                    }
                     if (this.contentDecoder == null) {
                         // No request entity is expected
                         // Ready to receive a new request
@@ -171,35 +276,43 @@ public class DefaultNHttpServerConnection
                     }
                 }
                 if (bytesRead == -1) {
-                    close();
+                    handler.endOfInput(this);
                 }
             }
-            if (this.contentDecoder != null) {
+            if (this.contentDecoder != null && (this.session.getEventMask() & SelectionKey.OP_READ) > 0) {
                 handler.inputReady(this, this.contentDecoder);
-                if (this.session.isClosed()) {
-                    return;
-                }
                 if (this.contentDecoder.isCompleted()) {
                     // Request entity received
                     // Ready to receive a new request
                     resetInput();
                 }
             }
-        } catch (IOException ex) {
-            handler.exception(this, ex);
-        } catch (HttpException ex) {
+        } catch (final HttpException ex) {
             resetInput();
             handler.exception(this, ex);
+        } catch (final Exception ex) {
+            handler.exception(this, ex);
         } finally {
             // Finally set buffered input flag
             this.hasBufferedInput = this.inbuf.hasData();
         }
     }
 
-    public void produceOutput(final NHttpServiceHandler handler) {
+    public void produceOutput(final NHttpServerEventHandler handler) {
         try {
+            if (this.status == ACTIVE) {
+                if (this.contentEncoder == null) {
+                    handler.responseReady(this);
+                }
+                if (this.contentEncoder != null) {
+                    handler.outputReady(this, this.contentEncoder);
+                    if (this.contentEncoder.isCompleted()) {
+                        resetOutput();
+                    }
+                }
+            }
             if (this.outbuf.hasData()) {
-                int bytesWritten = this.outbuf.flush(this.session.channel());
+                final int bytesWritten = this.outbuf.flush(this.session.channel());
                 if (bytesWritten > 0) {
                     this.outTransportMetrics.incrementBytesTransferred(bytesWritten);
                 }
@@ -209,28 +322,12 @@ public class DefaultNHttpServerConnection
                     this.session.close();
                     this.status = CLOSED;
                     resetOutput();
-                    return;
-                } else {
-                    if (this.contentEncoder != null) {
-                        handler.outputReady(this, this.contentEncoder);
-                        if (this.contentEncoder.isCompleted()) {
-                            resetOutput();
-                        }
-                    }
                 }
-
-                if (this.contentEncoder == null && !this.outbuf.hasData()) {
-                    if (this.status == CLOSING) {
-                        this.session.close();
-                        this.status = CLOSED;
-                    }
-                    if (this.status != CLOSED) {
-                        this.session.clearEvent(EventMask.WRITE);
-                        handler.responseReady(this);
-                    }
+                if (this.contentEncoder == null && this.status != CLOSED) {
+                    this.session.clearEvent(EventMask.WRITE);
                 }
             }
-        } catch (IOException ex) {
+        } catch (final Exception ex) {
             handler.exception(this, ex);
         } finally {
             // Finally set the buffered output flag
@@ -239,13 +336,12 @@ public class DefaultNHttpServerConnection
     }
 
     public void submitResponse(final HttpResponse response) throws IOException, HttpException {
-        if (response == null) {
-            throw new IllegalArgumentException("HTTP response may not be null");
-        }
+        Args.notNull(response, "HTTP response");
         assertNotClosed();
         if (this.response != null) {
             throw new HttpException("Response already submitted");
         }
+        onResponseSubmitted(response);
         this.responseWriter.write(response);
         this.hasBufferedOutput = this.outbuf.hasData();
 
@@ -264,37 +360,12 @@ public class DefaultNHttpServerConnection
         return this.response != null;
     }
 
-    public String toString() {
-        StringBuilder buf = new StringBuilder();
+    public void consumeInput(final NHttpServiceHandler handler) {
+        consumeInput(new NHttpServerEventHandlerAdaptor(handler));
+    }
 
-        final SocketAddress remoteAddress = this.session.getRemoteAddress();
-        final SocketAddress localAddress  = this.session.getLocalAddress();
-
-        if (remoteAddress == null || localAddress == null) {
-            return "[CLOSED]";
-        }
-
-        if (remoteAddress instanceof InetSocketAddress &&
-            localAddress instanceof InetSocketAddress) {
-
-            final InetSocketAddress remote = ((InetSocketAddress) remoteAddress);
-            final InetSocketAddress local  = ((InetSocketAddress) localAddress);
-
-            buf.append(remote.getAddress() != null ? remote.getAddress().getHostAddress() : remote.getAddress())
-            .append(':')
-            .append(remote.getPort())
-            .append("->")
-            .append(local.getAddress() != null ? local.getAddress().getHostAddress() : local.getAddress())
-            .append(':')
-            .append(local.getPort());
-
-        } else {
-            buf.append(remoteAddress)
-            .append("->")
-            .append(localAddress);
-        }
-
-        return buf.toString();
+    public void produceOutput(final NHttpServiceHandler handler) {
+        produceOutput(new NHttpServerEventHandlerAdaptor(handler));
     }
 
 }
