@@ -30,11 +30,17 @@ package org.apache.http.impl.nio;
 import java.io.IOException;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
+import java.net.Socket;
 import java.net.SocketAddress;
 import java.nio.channels.ReadableByteChannel;
 import java.nio.channels.WritableByteChannel;
+import java.nio.charset.Charset;
+import java.nio.charset.CharsetDecoder;
+import java.nio.charset.CharsetEncoder;
+import java.nio.charset.CodingErrorAction;
 
 import org.apache.http.ConnectionClosedException;
+import org.apache.http.Consts;
 import org.apache.http.Header;
 import org.apache.http.HttpConnectionMetrics;
 import org.apache.http.HttpEntity;
@@ -43,15 +49,13 @@ import org.apache.http.HttpInetConnection;
 import org.apache.http.HttpMessage;
 import org.apache.http.HttpRequest;
 import org.apache.http.HttpResponse;
+import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.entity.BasicHttpEntity;
 import org.apache.http.entity.ContentLengthStrategy;
 import org.apache.http.impl.HttpConnectionMetricsImpl;
 import org.apache.http.impl.entity.LaxContentLengthStrategy;
 import org.apache.http.impl.entity.StrictContentLengthStrategy;
 import org.apache.http.impl.io.HttpTransportMetricsImpl;
-import org.apache.http.nio.ContentDecoder;
-import org.apache.http.nio.ContentEncoder;
-import org.apache.http.nio.NHttpConnection;
 import org.apache.http.impl.nio.codecs.ChunkDecoder;
 import org.apache.http.impl.nio.codecs.ChunkEncoder;
 import org.apache.http.impl.nio.codecs.IdentityDecoder;
@@ -61,39 +65,42 @@ import org.apache.http.impl.nio.codecs.LengthDelimitedEncoder;
 import org.apache.http.impl.nio.reactor.SessionInputBufferImpl;
 import org.apache.http.impl.nio.reactor.SessionOutputBufferImpl;
 import org.apache.http.io.HttpTransportMetrics;
+import org.apache.http.nio.ContentDecoder;
+import org.apache.http.nio.ContentEncoder;
+import org.apache.http.nio.NHttpConnection;
 import org.apache.http.nio.reactor.EventMask;
 import org.apache.http.nio.reactor.IOSession;
 import org.apache.http.nio.reactor.SessionBufferStatus;
 import org.apache.http.nio.reactor.SessionInputBuffer;
 import org.apache.http.nio.reactor.SessionOutputBuffer;
+import org.apache.http.nio.reactor.SocketAccessor;
 import org.apache.http.nio.util.ByteBufferAllocator;
-import org.apache.http.params.HttpConnectionParams;
+import org.apache.http.params.CoreConnectionPNames;
+import org.apache.http.params.CoreProtocolPNames;
 import org.apache.http.params.HttpParams;
 import org.apache.http.protocol.HTTP;
 import org.apache.http.protocol.HttpContext;
+import org.apache.http.util.Args;
+import org.apache.http.util.CharsetUtils;
+import org.apache.http.util.NetUtils;
 
 /**
- * This class serves as a base for all {@link NHttpConnection} implementations
- * and implements functionality common to both client and server
- * HTTP connections.
- * <p>
- * The following parameters can be used to customize the behavior of this
- * class:
- * <ul>
- *  <li>{@link org.apache.http.params.CoreProtocolPNames#HTTP_ELEMENT_CHARSET}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#SOCKET_BUFFER_SIZE}</li>
- * </ul>
+ * This class serves as a base for all {@link NHttpConnection} implementations and provides
+ * functionality common to both client and server HTTP connections.
  *
  * @since 4.0
  */
+@SuppressWarnings("deprecation")
+@NotThreadSafe
 public class NHttpConnectionBase
-        implements NHttpConnection, HttpInetConnection, SessionBufferStatus {
+        implements NHttpConnection, HttpInetConnection, SessionBufferStatus, SocketAccessor {
 
     protected final ContentLengthStrategy incomingContentStrategy;
     protected final ContentLengthStrategy outgoingContentStrategy;
 
     protected final SessionInputBufferImpl inbuf;
     protected final SessionOutputBufferImpl outbuf;
+    private final int fragmentSizeHint;
 
     protected final HttpTransportMetricsImpl inTransportMetrics;
     protected final HttpTransportMetricsImpl outTransportMetrics;
@@ -117,30 +124,50 @@ public class NHttpConnectionBase
      * @param session the underlying I/O session.
      * @param allocator byte buffer allocator.
      * @param params HTTP parameters.
+     *
+     * @deprecated (4.3) use
+     *   {@link NHttpConnectionBase#NHttpConnectionBase(IOSession, int, int, ByteBufferAllocator,
+     *   CharsetDecoder, CharsetEncoder, ContentLengthStrategy, ContentLengthStrategy)}
      */
+    @Deprecated
     public NHttpConnectionBase(
             final IOSession session,
             final ByteBufferAllocator allocator,
             final HttpParams params) {
         super();
-        if (session == null) {
-            throw new IllegalArgumentException("I/O session may not be null");
-        }
-        if (params == null) {
-            throw new IllegalArgumentException("HTTP params may not be null");
-        }
+        Args.notNull(session, "I/O session");
+        Args.notNull(params, "HTTP params");
 
-        int buffersize = HttpConnectionParams.getSocketBufferSize(params);
+        int buffersize = params.getIntParameter(CoreConnectionPNames.SOCKET_BUFFER_SIZE, -1);
+        if (buffersize <= 0) {
+            buffersize = 4096;
+        }
         int linebuffersize = buffersize;
         if (linebuffersize > 512) {
             linebuffersize = 512;
         }
 
-        this.inbuf = new SessionInputBufferImpl(buffersize, linebuffersize, allocator, params);
-        this.outbuf = new SessionOutputBufferImpl(buffersize, linebuffersize, allocator, params);
+        CharsetDecoder decoder = null;
+        CharsetEncoder encoder = null;
+        Charset charset = CharsetUtils.lookup(
+                (String) params.getParameter(CoreProtocolPNames.HTTP_ELEMENT_CHARSET));
+        if (charset != null) {
+            charset = Consts.ASCII;
+            decoder = charset.newDecoder();
+            encoder = charset.newEncoder();
+            final CodingErrorAction malformedCharAction = (CodingErrorAction) params.getParameter(
+                    CoreProtocolPNames.HTTP_MALFORMED_INPUT_ACTION);
+            final CodingErrorAction unmappableCharAction = (CodingErrorAction) params.getParameter(
+                    CoreProtocolPNames.HTTP_UNMAPPABLE_INPUT_ACTION);
+            decoder.onMalformedInput(malformedCharAction).onUnmappableCharacter(unmappableCharAction);
+            encoder.onMalformedInput(malformedCharAction).onUnmappableCharacter(unmappableCharAction);
+        }
+        this.inbuf = new SessionInputBufferImpl(buffersize, linebuffersize, decoder, allocator);
+        this.outbuf = new SessionOutputBufferImpl(buffersize, linebuffersize, encoder, allocator);
+        this.fragmentSizeHint = buffersize;
 
-        this.incomingContentStrategy = new LaxContentLengthStrategy();
-        this.outgoingContentStrategy = new StrictContentLengthStrategy();
+        this.incomingContentStrategy = createIncomingContentStrategy();
+        this.outgoingContentStrategy = createOutgoingContentStrategy();
 
         this.inTransportMetrics = createTransportMetrics();
         this.outTransportMetrics = createTransportMetrics();
@@ -152,6 +179,57 @@ public class NHttpConnectionBase
         this.status = ACTIVE;
     }
 
+    /**
+     * Creates new instance NHttpConnectionBase given the underlying I/O session.
+     *
+     * @param session the underlying I/O session.
+     * @param buffersize buffer size. Must be a positive number.
+     * @param fragmentSizeHint fragment size hint.
+     * @param allocator memory allocator.
+     *   If <code>null</code> {@link org.apache.http.nio.util.HeapByteBufferAllocator#INSTANCE}
+     *   will be used.
+     * @param chardecoder decoder to be used for decoding HTTP protocol elements.
+     *   If <code>null</code> simple type cast will be used for byte to char conversion.
+     * @param charencoder encoder to be used for encoding HTTP protocol elements.
+     *   If <code>null</code> simple type cast will be used for char to byte conversion.
+     * @param incomingContentStrategy incoming content length strategy. If <code>null</code>
+     *   {@link LaxContentLengthStrategy#INSTANCE} will be used.
+     * @param outgoingContentStrategy outgoing content length strategy. If <code>null</code>
+     *   {@link StrictContentLengthStrategy#INSTANCE} will be used.
+     *
+     * @since 4.3
+     */
+    protected NHttpConnectionBase(
+            final IOSession session,
+            final int buffersize,
+            final int fragmentSizeHint,
+            final ByteBufferAllocator allocator,
+            final CharsetDecoder chardecoder,
+            final CharsetEncoder charencoder,
+            final ContentLengthStrategy incomingContentStrategy,
+            final ContentLengthStrategy outgoingContentStrategy) {
+        Args.notNull(session, "I/O session");
+        Args.positive(buffersize, "Buffer size");
+        int linebuffersize = buffersize;
+        if (linebuffersize > 512) {
+            linebuffersize = 512;
+        }
+        this.inbuf = new SessionInputBufferImpl(buffersize, linebuffersize, chardecoder, allocator);
+        this.outbuf = new SessionOutputBufferImpl(buffersize, linebuffersize, charencoder, allocator);
+        this.fragmentSizeHint = fragmentSizeHint >= 0 ? fragmentSizeHint : buffersize;
+
+        this.inTransportMetrics = new HttpTransportMetricsImpl();
+        this.outTransportMetrics = new HttpTransportMetricsImpl();
+        this.connMetrics = new HttpConnectionMetricsImpl(this.inTransportMetrics, this.outTransportMetrics);
+        this.incomingContentStrategy = incomingContentStrategy != null ? incomingContentStrategy :
+            LaxContentLengthStrategy.INSTANCE;
+        this.outgoingContentStrategy = outgoingContentStrategy != null ? outgoingContentStrategy :
+            StrictContentLengthStrategy.INSTANCE;
+
+        setSession(session);
+        this.status = ACTIVE;
+    }
+
     private void setSession(final IOSession session) {
         this.session = session;
         this.context = new SessionHttpContext(this.session);
@@ -160,15 +238,53 @@ public class NHttpConnectionBase
     }
 
     /**
-     * @since 4.1
+     * Binds the connection to a different {@link IOSession}. This may be necessary
+     * when the underlying I/O session gets upgraded with SSL/TLS encryption.
+     *
+     * @since 4.2
      */
+    protected void bind(final IOSession session) {
+        Args.notNull(session, "I/O session");
+        this.session.setBufferStatus(null);
+        setSession(session);
+    }
+
+    /**
+     * @since 4.2
+     *
+     * @deprecated (4.3) use constructor.
+     */
+    @Deprecated
+    protected ContentLengthStrategy createIncomingContentStrategy() {
+        return new LaxContentLengthStrategy();
+    }
+
+    /**
+     * @since 4.2
+     *
+     * @deprecated (4.3) use constructor.
+     */
+    @Deprecated
+    protected ContentLengthStrategy createOutgoingContentStrategy() {
+        return new StrictContentLengthStrategy();
+    }
+
+    /**
+     * @since 4.1
+     *
+     * @deprecated (4.3) no longer used.
+     */
+    @Deprecated
     protected HttpTransportMetricsImpl createTransportMetrics() {
         return new HttpTransportMetricsImpl();
     }
 
     /**
      * @since 4.1
+     *
+     * @deprecated (4.3) use decorator to add additional metrics.
      */
+    @Deprecated
     protected HttpConnectionMetricsImpl createConnectionMetrics(
             final HttpTransportMetrics inTransportMetric,
             final HttpTransportMetrics outTransportMetric) {
@@ -217,8 +333,8 @@ public class NHttpConnectionBase
      * @throws HttpException in case of an HTTP protocol violation.
      */
     protected HttpEntity prepareDecoder(final HttpMessage message) throws HttpException {
-        BasicHttpEntity entity = new BasicHttpEntity();
-        long len = this.incomingContentStrategy.determineLength(message);
+        final BasicHttpEntity entity = new BasicHttpEntity();
+        final long len = this.incomingContentStrategy.determineLength(message);
         this.contentDecoder = createContentDecoder(
                 len,
                 this.session.channel(),
@@ -235,11 +351,11 @@ public class NHttpConnectionBase
             entity.setContentLength(len);
         }
 
-        Header contentTypeHeader = message.getFirstHeader(HTTP.CONTENT_TYPE);
+        final Header contentTypeHeader = message.getFirstHeader(HTTP.CONTENT_TYPE);
         if (contentTypeHeader != null) {
             entity.setContentType(contentTypeHeader);
         }
-        Header contentEncodingHeader = message.getFirstHeader(HTTP.CONTENT_ENCODING);
+        final Header contentEncodingHeader = message.getFirstHeader(HTTP.CONTENT_ENCODING);
         if (contentEncodingHeader != null) {
             entity.setContentEncoding(contentEncodingHeader);
         }
@@ -281,7 +397,7 @@ public class NHttpConnectionBase
      * @throws HttpException in case of an HTTP protocol violation.
      */
     protected void prepareEncoder(final HttpMessage message) throws HttpException {
-        long len = this.outgoingContentStrategy.determineLength(message);
+        final long len = this.outgoingContentStrategy.determineLength(message);
         this.contentEncoder = createContentEncoder(
                 len,
                 this.session.channel(),
@@ -308,11 +424,11 @@ public class NHttpConnectionBase
             final SessionOutputBuffer buffer,
             final HttpTransportMetricsImpl metrics) {
         if (len == ContentLengthStrategy.CHUNKED) {
-            return new ChunkEncoder(channel, buffer, metrics);
+            return new ChunkEncoder(channel, buffer, metrics, this.fragmentSizeHint);
         } else if (len == ContentLengthStrategy.IDENTITY) {
-            return new IdentityEncoder(channel, buffer, metrics);
+            return new IdentityEncoder(channel, buffer, metrics, this.fragmentSizeHint);
         } else {
-            return new LengthDelimitedEncoder(channel, buffer, metrics, len);
+            return new LengthDelimitedEncoder(channel, buffer, metrics, len, this.fragmentSizeHint);
         }
     }
 
@@ -358,7 +474,7 @@ public class NHttpConnectionBase
     }
 
     public InetAddress getLocalAddress() {
-        SocketAddress address = this.session.getLocalAddress();
+        final SocketAddress address = this.session.getLocalAddress();
         if (address instanceof InetSocketAddress) {
             return ((InetSocketAddress) address).getAddress();
         } else {
@@ -367,7 +483,7 @@ public class NHttpConnectionBase
     }
 
     public int getLocalPort() {
-        SocketAddress address = this.session.getLocalAddress();
+        final SocketAddress address = this.session.getLocalAddress();
         if (address instanceof InetSocketAddress) {
             return ((InetSocketAddress) address).getPort();
         } else {
@@ -376,7 +492,7 @@ public class NHttpConnectionBase
     }
 
     public InetAddress getRemoteAddress() {
-        SocketAddress address = this.session.getRemoteAddress();
+        final SocketAddress address = this.session.getRemoteAddress();
         if (address instanceof InetSocketAddress) {
             return ((InetSocketAddress) address).getAddress();
         } else {
@@ -385,7 +501,7 @@ public class NHttpConnectionBase
     }
 
     public int getRemotePort() {
-        SocketAddress address = this.session.getRemoteAddress();
+        final SocketAddress address = this.session.getRemoteAddress();
         if (address instanceof InetSocketAddress) {
             return ((InetSocketAddress) address).getPort();
         } else {
@@ -393,7 +509,7 @@ public class NHttpConnectionBase
         }
     }
 
-    public void setSocketTimeout(int timeout) {
+    public void setSocketTimeout(final int timeout) {
         this.session.setSocketTimeout(timeout);
     }
 
@@ -412,11 +528,36 @@ public class NHttpConnectionBase
 
     @Override
     public String toString() {
-        StringBuilder buf = new StringBuilder();
-        buf.append(this.session.getLocalAddress())
-            .append("<->")
-            .append(this.session.getRemoteAddress());
-        return buf.toString();
+        final StringBuilder buffer = new StringBuilder();
+        final SocketAddress remoteAddress = this.session.getRemoteAddress();
+        final SocketAddress localAddress = this.session.getLocalAddress();
+        if (remoteAddress != null && localAddress != null) {
+            NetUtils.formatAddress(buffer, localAddress);
+            buffer.append("<->");
+            NetUtils.formatAddress(buffer, remoteAddress);
+        }
+        buffer.append("[");
+        switch (this.status) {
+        case ACTIVE:
+            buffer.append("ACTIVE");
+            break;
+        case CLOSING:
+            buffer.append("CLOSING");
+            break;
+        case CLOSED:
+            buffer.append("CLOSED");
+            break;
+        }
+        buffer.append("]");
+        return buffer.toString();
+    }
+
+    public Socket getSocket() {
+        if (this.session instanceof SocketAccessor) {
+            return ((SocketAccessor) this.session).getSocket();
+        } else {
+            return null;
+        }
     }
 
 }
