@@ -34,29 +34,28 @@ import java.util.List;
 
 import org.apache.http.HttpException;
 import org.apache.http.HttpMessage;
+import org.apache.http.MessageConstraintException;
 import org.apache.http.ParseException;
 import org.apache.http.ProtocolException;
-import org.apache.http.message.LineParser;
+import org.apache.http.annotation.NotThreadSafe;
+import org.apache.http.config.MessageConstraints;
 import org.apache.http.message.BasicLineParser;
+import org.apache.http.message.LineParser;
 import org.apache.http.nio.NHttpMessageParser;
 import org.apache.http.nio.reactor.SessionInputBuffer;
-import org.apache.http.params.CoreConnectionPNames;
+import org.apache.http.params.HttpParamConfig;
 import org.apache.http.params.HttpParams;
+import org.apache.http.util.Args;
 import org.apache.http.util.CharArrayBuffer;
 
 /**
  * Abstract {@link NHttpMessageParser} that serves as a base for all message
  * parser implementations.
- * <p>
- * The following parameters can be used to customize the behavior of this
- * class:
- * <ul>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#MAX_HEADER_COUNT}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#MAX_LINE_LENGTH}</li>
- * </ul>
  *
  * @since 4.0
  */
+@SuppressWarnings("deprecation")
+@NotThreadSafe
 public abstract class AbstractMessageParser<T extends HttpMessage> implements NHttpMessageParser<T> {
 
     private final SessionInputBuffer sessionBuffer;
@@ -72,34 +71,58 @@ public abstract class AbstractMessageParser<T extends HttpMessage> implements NH
     private CharArrayBuffer lineBuf;
     private final List<CharArrayBuffer> headerBufs;
 
-    private int maxLineLen = -1;
-    private int maxHeaderCount = -1;
     protected final LineParser lineParser;
+    private final MessageConstraints constraints;
 
     /**
      * Creates an instance of this class.
      *
      * @param buffer the session input buffer.
-     * @param parser the line parser.
+     * @param lineParser the line parser.
      * @param params HTTP parameters.
+     *
+     * @deprecated (4.3) use
+     *   {@link AbstractMessageParser#AbstractMessageParser(SessionInputBuffer, LineParser,
+     *     MessageConstraints)}
      */
-    public AbstractMessageParser(final SessionInputBuffer buffer, final LineParser parser, final HttpParams params) {
+    @Deprecated
+    public AbstractMessageParser(
+            final SessionInputBuffer buffer,
+            final LineParser lineParser,
+            final HttpParams params) {
         super();
-        if (buffer == null) {
-            throw new IllegalArgumentException("Session input buffer may not be null");
-        }
-        if (params == null) {
-            throw new IllegalArgumentException("HTTP parameters may not be null");
-        }
+        Args.notNull(buffer, "Session input buffer");
+        Args.notNull(params, "HTTP parameters");
         this.sessionBuffer = buffer;
         this.state = READ_HEAD_LINE;
         this.endOfStream = false;
         this.headerBufs = new ArrayList<CharArrayBuffer>();
-        this.maxLineLen = params.getIntParameter(
-                CoreConnectionPNames.MAX_LINE_LENGTH, -1);
-        this.maxHeaderCount = params.getIntParameter(
-                CoreConnectionPNames.MAX_HEADER_COUNT, -1);
-        this.lineParser = (parser != null) ? parser : BasicLineParser.DEFAULT;
+        this.constraints = HttpParamConfig.getMessageConstraints(params);
+        this.lineParser = (lineParser != null) ? lineParser : BasicLineParser.INSTANCE;
+    }
+
+    /**
+     * Creates an instance of AbstractMessageParser.
+     *
+     * @param buffer the session input buffer.
+     * @param lineParser the line parser. If <code>null</code> {@link BasicLineParser#INSTANCE}
+     *   will be used.
+     * @param constraints Message constraints. If <code>null</code>
+     *   {@link MessageConstraints#DEFAULT} will be used.
+     *
+     * @since 4.3
+     */
+    public AbstractMessageParser(
+            final SessionInputBuffer buffer,
+            final LineParser lineParser,
+            final MessageConstraints constraints) {
+        super();
+        this.sessionBuffer = Args.notNull(buffer, "Session input buffer");
+        this.lineParser = lineParser != null ? lineParser : BasicLineParser.INSTANCE;
+        this.constraints = constraints != null ? constraints : MessageConstraints.DEFAULT;
+        this.headerBufs = new ArrayList<CharArrayBuffer>();
+        this.state = READ_HEAD_LINE;
+        this.endOfStream = false;
     }
 
     public void reset() {
@@ -110,7 +133,7 @@ public abstract class AbstractMessageParser<T extends HttpMessage> implements NH
     }
 
     public int fillBuffer(final ReadableByteChannel channel) throws IOException {
-        int bytesRead = this.sessionBuffer.fill(channel);
+        final int bytesRead = this.sessionBuffer.fill(channel);
         if (bytesRead == -1) {
             this.endOfStream = true;
         }
@@ -134,22 +157,22 @@ public abstract class AbstractMessageParser<T extends HttpMessage> implements NH
     }
 
     private void parseHeader() throws IOException {
-        CharArrayBuffer current = this.lineBuf;
-        int count = this.headerBufs.size();
+        final CharArrayBuffer current = this.lineBuf;
+        final int count = this.headerBufs.size();
         if ((this.lineBuf.charAt(0) == ' ' || this.lineBuf.charAt(0) == '\t') && count > 0) {
             // Handle folded header line
-            CharArrayBuffer previous = this.headerBufs.get(count - 1);
+            final CharArrayBuffer previous = this.headerBufs.get(count - 1);
             int i = 0;
             while (i < current.length()) {
-                char ch = current.charAt(i);
+                final char ch = current.charAt(i);
                 if (ch != ' ' && ch != '\t') {
                     break;
                 }
                 i++;
             }
-            if (this.maxLineLen > 0
-                    && previous.length() + 1 + current.length() - i > this.maxLineLen) {
-                throw new IOException("Maximum line length limit exceeded");
+            final int maxLineLen = this.constraints.getMaxLineLength();
+            if (maxLineLen > 0 && previous.length() + 1 + current.length() - i > maxLineLen) {
+                throw new MessageConstraintException("Maximum line length limit exceeded");
             }
             previous.append(' ');
             previous.append(current, i, current.length() - i);
@@ -166,11 +189,12 @@ public abstract class AbstractMessageParser<T extends HttpMessage> implements NH
             } else {
                 this.lineBuf.clear();
             }
-            boolean lineComplete = this.sessionBuffer.readLine(this.lineBuf, this.endOfStream);
-            if (this.maxLineLen > 0 &&
-                    (this.lineBuf.length() > this.maxLineLen ||
-                            (!lineComplete && this.sessionBuffer.length() > this.maxLineLen))) {
-                throw new IOException("Maximum line length limit exceeded");
+            final boolean lineComplete = this.sessionBuffer.readLine(this.lineBuf, this.endOfStream);
+            final int maxLineLen = this.constraints.getMaxLineLength();
+            if (maxLineLen > 0 &&
+                    (this.lineBuf.length() > maxLineLen ||
+                            (!lineComplete && this.sessionBuffer.length() > maxLineLen))) {
+                throw new MessageConstraintException("Maximum line length limit exceeded");
             }
             if (!lineComplete) {
                 break;
@@ -180,15 +204,16 @@ public abstract class AbstractMessageParser<T extends HttpMessage> implements NH
             case READ_HEAD_LINE:
                 try {
                     parseHeadLine();
-                } catch (ParseException px) {
+                } catch (final ParseException px) {
                     throw new ProtocolException(px.getMessage(), px);
                 }
                 this.state = READ_HEADERS;
                 break;
             case READ_HEADERS:
                 if (this.lineBuf.length() > 0) {
-                    if (this.maxHeaderCount > 0 && headerBufs.size() >= this.maxHeaderCount) {
-                        throw new IOException("Maximum header count exceeded");
+                    final int maxHeaderCount = this.constraints.getMaxHeaderCount();
+                    if (maxHeaderCount > 0 && headerBufs.size() >= maxHeaderCount) {
+                        throw new MessageConstraintException("Maximum header count exceeded");
                     }
 
                     parseHeader();
@@ -202,11 +227,10 @@ public abstract class AbstractMessageParser<T extends HttpMessage> implements NH
             }
         }
         if (this.state == COMPLETED) {
-            for (int i = 0; i < this.headerBufs.size(); i++) {
-                CharArrayBuffer buffer = this.headerBufs.get(i);
+            for (final CharArrayBuffer buffer : this.headerBufs) {
                 try {
                     this.message.addHeader(lineParser.parseHeader(buffer));
-                } catch (ParseException ex) {
+                } catch (final ParseException ex) {
                     throw new ProtocolException(ex.getMessage(), ex);
                 }
             }
