@@ -32,13 +32,15 @@ import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
 import java.nio.channels.ReadableByteChannel;
 
+import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.impl.io.HttpTransportMetricsImpl;
 import org.apache.http.nio.FileContentDecoder;
 import org.apache.http.nio.reactor.SessionInputBuffer;
+import org.apache.http.util.Args;
 
 /**
  * Content decoder that reads data without any transformation. The end of the
- * content entity is demarcated by closing the underlying connection
+ * content entity is delineated by closing the underlying connection
  * (EOF condition). Entities transferred using this input stream can be of
  * unlimited length.
  * <p>
@@ -48,6 +50,7 @@ import org.apache.http.nio.reactor.SessionInputBuffer;
  *
  * @since 4.0
  */
+@NotThreadSafe
 public class IdentityDecoder extends AbstractContentDecoder
         implements FileContentDecoder {
 
@@ -65,26 +68,21 @@ public class IdentityDecoder extends AbstractContentDecoder
      * some other means to know all the necessary data has been read and want to
      * reuse the underlying connection for more messages.
      */
-    public void setCompleted(boolean completed) {
+    public void setCompleted(final boolean completed) {
         this.completed = completed;
     }
 
     public int read(final ByteBuffer dst) throws IOException {
-        if (dst == null) {
-            throw new IllegalArgumentException("Byte buffer may not be null");
-        }
+        Args.notNull(dst, "Byte buffer");
         if (this.completed) {
             return -1;
         }
 
-        int bytesRead;
+        final int bytesRead;
         if (this.buffer.hasData()) {
             bytesRead = this.buffer.read(dst);
         } else {
-            bytesRead = this.channel.read(dst);
-            if (bytesRead > 0) {
-                this.metrics.incrementBytesTransferred(bytesRead);
-            }
+            bytesRead = readFromChannel(dst);
         }
         if (bytesRead == -1) {
             this.completed = true;
@@ -94,8 +92,8 @@ public class IdentityDecoder extends AbstractContentDecoder
 
     public long transfer(
             final FileChannel dst,
-            long position,
-            long count) throws IOException {
+            final long position,
+            final long count) throws IOException {
 
         if (dst == null) {
             return 0;
@@ -110,14 +108,13 @@ public class IdentityDecoder extends AbstractContentDecoder
             bytesRead = this.buffer.read(dst);
         } else {
             if (this.channel.isOpen()) {
-                if(dst.size() < position)
-                    throw new IOException("FileChannel.size() [" + dst.size() +
-                                          "] < position [" + position +
-                                          "].  Please grow the file before writing.");
-
+                if (position > dst.size()) {
+                    throw new IOException("Position past end of file [" + position +
+                            " > " + dst.size() + "]");
+                }
                 bytesRead = dst.transferFrom(this.channel, position, count);
-                if (bytesRead == 0) {
-                    bytesRead = buffer.fill(this.channel);
+                if (count > 0 && bytesRead == 0) {
+                    bytesRead = this.buffer.fill(this.channel);
                 }
             } else {
                 bytesRead = -1;
@@ -134,11 +131,11 @@ public class IdentityDecoder extends AbstractContentDecoder
 
     @Override
     public String toString() {
-        StringBuffer buffer = new StringBuffer();
-        buffer.append("[identity; completed: ");
-        buffer.append(this.completed);
-        buffer.append("]");
-        return buffer.toString();
+        final StringBuilder sb = new StringBuilder();
+        sb.append("[identity; completed: ");
+        sb.append(this.completed);
+        sb.append("]");
+        return sb.toString();
     }
 
 }
