@@ -37,15 +37,17 @@ import java.nio.channels.Selector;
 import java.nio.channels.SocketChannel;
 import java.util.Collections;
 import java.util.HashSet;
-import java.util.Iterator;
 import java.util.Queue;
 import java.util.Set;
 import java.util.concurrent.ConcurrentLinkedQueue;
 
+import org.apache.http.annotation.ThreadSafe;
 import org.apache.http.nio.reactor.IOReactor;
 import org.apache.http.nio.reactor.IOReactorException;
 import org.apache.http.nio.reactor.IOReactorStatus;
 import org.apache.http.nio.reactor.IOSession;
+import org.apache.http.util.Args;
+import org.apache.http.util.Asserts;
 
 /**
  * Generic implementation of {@link IOReactor} that can used as a subclass
@@ -54,6 +56,7 @@ import org.apache.http.nio.reactor.IOSession;
  *
  * @since 4.0
  */
+@ThreadSafe // public methods only
 public abstract class AbstractIOReactor implements IOReactor {
 
     private volatile IOReactorStatus status;
@@ -73,7 +76,7 @@ public abstract class AbstractIOReactor implements IOReactor {
      * @param selectTimeout the select timeout.
      * @throws IOReactorException in case if a non-recoverable I/O error.
      */
-    public AbstractIOReactor(long selectTimeout) throws IOReactorException {
+    public AbstractIOReactor(final long selectTimeout) throws IOReactorException {
         this(selectTimeout, false);
     }
 
@@ -87,11 +90,9 @@ public abstract class AbstractIOReactor implements IOReactor {
      *
      * @since 4.1
      */
-    public AbstractIOReactor(long selectTimeout, boolean interestOpsQueueing) throws IOReactorException {
+    public AbstractIOReactor(final long selectTimeout, final boolean interestOpsQueueing) throws IOReactorException {
         super();
-        if (selectTimeout <= 0) {
-            throw new IllegalArgumentException("Select timeout may not be negative or zero");
-        }
+        Args.positive(selectTimeout, "Select timeout");
         this.selectTimeout = selectTimeout;
         this.interestOpsQueueing = interestOpsQueueing;
         this.sessions = Collections.synchronizedSet(new HashSet<IOSession>());
@@ -100,7 +101,7 @@ public abstract class AbstractIOReactor implements IOReactor {
         this.newChannels = new ConcurrentLinkedQueue<ChannelEntry>();
         try {
             this.selector = Selector.open();
-        } catch (IOException ex) {
+        } catch (final IOException ex) {
             throw new IOReactorException("Failure opening selector", ex);
         }
         this.statusMutex = new Object();
@@ -217,9 +218,7 @@ public abstract class AbstractIOReactor implements IOReactor {
      * @param channelEntry the channel entry.
      */
     public void addChannel(final ChannelEntry channelEntry) {
-        if (channelEntry == null) {
-            throw new IllegalArgumentException("Channel entry may not be null");
-        }
+        Args.notNull(channelEntry, "Channel entry");
         this.newChannels.add(channelEntry);
         this.selector.wakeup();
     }
@@ -252,12 +251,12 @@ public abstract class AbstractIOReactor implements IOReactor {
         try {
             for (;;) {
 
-                int readyCount;
+                final int readyCount;
                 try {
                     readyCount = this.selector.select(this.selectTimeout);
-                } catch (InterruptedIOException ex) {
+                } catch (final InterruptedIOException ex) {
                     throw ex;
-                } catch (IOException ex) {
+                } catch (final IOException ex) {
                     throw new IOReactorException("Unexpected selector failure", ex);
                 }
 
@@ -302,7 +301,7 @@ public abstract class AbstractIOReactor implements IOReactor {
 
             }
 
-        } catch (ClosedSelectorException ex) {
+        } catch (final ClosedSelectorException ignore) {
         } finally {
             hardShutdown();
             synchronized (this.statusMutex) {
@@ -312,9 +311,8 @@ public abstract class AbstractIOReactor implements IOReactor {
     }
 
     private void processEvents(final Set<SelectionKey> selectedKeys) {
-        for (Iterator<SelectionKey> it = selectedKeys.iterator(); it.hasNext(); ) {
+        for (final SelectionKey key : selectedKeys) {
 
-            SelectionKey key = it.next();
             processEvent(key);
 
         }
@@ -327,7 +325,7 @@ public abstract class AbstractIOReactor implements IOReactor {
      * @param key the selection key that triggered an event.
      */
     protected void processEvent(final SelectionKey key) {
-        IOSessionImpl session = (IOSessionImpl) key.attachment();
+        final IOSessionImpl session = (IOSessionImpl) key.attachment();
         try {
             if (key.isAcceptable()) {
                 acceptable(key);
@@ -343,7 +341,7 @@ public abstract class AbstractIOReactor implements IOReactor {
                 session.resetLastWrite();
                 writable(key);
             }
-        } catch (CancelledKeyException ex) {
+        } catch (final CancelledKeyException ex) {
             queueClosedSession(session);
             key.attach(null);
         }
@@ -364,27 +362,27 @@ public abstract class AbstractIOReactor implements IOReactor {
         ChannelEntry entry;
         while ((entry = this.newChannels.poll()) != null) {
 
-            SocketChannel channel;
-            SelectionKey key;
+            final SocketChannel channel;
+            final SelectionKey key;
             try {
                 channel = entry.getChannel();
                 channel.configureBlocking(false);
                 key = channel.register(this.selector, SelectionKey.OP_READ);
-            } catch (ClosedChannelException ex) {
-                SessionRequestImpl sessionRequest = entry.getSessionRequest();
+            } catch (final ClosedChannelException ex) {
+                final SessionRequestImpl sessionRequest = entry.getSessionRequest();
                 if (sessionRequest != null) {
                     sessionRequest.failed(ex);
                 }
                 return;
 
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 throw new IOReactorException("Failure registering channel " +
                         "with the selector", ex);
             }
 
-            SessionClosedCallback sessionClosedCallback = new SessionClosedCallback() {
+            final SessionClosedCallback sessionClosedCallback = new SessionClosedCallback() {
 
-                public void sessionClosed(IOSession session) {
+                public void sessionClosed(final IOSession session) {
                     queueClosedSession(session);
                 }
 
@@ -401,29 +399,32 @@ public abstract class AbstractIOReactor implements IOReactor {
                 };
             }
 
-            IOSession session = new IOSessionImpl(key, interestOpsCallback, sessionClosedCallback);
-
-            int timeout = 0;
+            final IOSession session;
             try {
-                timeout = channel.socket().getSoTimeout();
-            } catch (IOException ex) {
-                // Very unlikely to happen and is not fatal
-                // as the protocol layer is expected to overwrite
-                // this value anyways
+                session = new IOSessionImpl(key, interestOpsCallback, sessionClosedCallback);
+                int timeout = 0;
+                try {
+                    timeout = channel.socket().getSoTimeout();
+                } catch (final IOException ex) {
+                    // Very unlikely to happen and is not fatal
+                    // as the protocol layer is expected to overwrite
+                    // this value anyways
+                }
+
+                session.setAttribute(IOSession.ATTACHMENT_KEY, entry.getAttachment());
+                session.setSocketTimeout(timeout);
+            } catch (final CancelledKeyException ex) {
+                continue;
             }
-
-            session.setAttribute(IOSession.ATTACHMENT_KEY, entry.getAttachment());
-            session.setSocketTimeout(timeout);
-            this.sessions.add(session);
-
             try {
-                SessionRequestImpl sessionRequest = entry.getSessionRequest();
+                this.sessions.add(session);
+                final SessionRequestImpl sessionRequest = entry.getSessionRequest();
                 if (sessionRequest != null) {
                     sessionRequest.completed(session);
                 }
                 key.attach(session);
                 sessionCreated(key, session);
-            } catch (CancelledKeyException ex) {
+            } catch (final CancelledKeyException ex) {
                 queueClosedSession(session);
                 key.attach(null);
             }
@@ -436,7 +437,7 @@ public abstract class AbstractIOReactor implements IOReactor {
             if (this.sessions.remove(session)) {
                 try {
                     sessionClosed(session);
-                } catch (CancelledKeyException ex) {
+                } catch (final CancelledKeyException ex) {
                     // ignore and move on
                 }
             }
@@ -451,8 +452,8 @@ public abstract class AbstractIOReactor implements IOReactor {
         InterestOpEntry entry;
         while ((entry = this.interestOpsQueue.poll()) != null) {
             // obtain the operation's details
-            SelectionKey key = entry.getSelectionKey();
-            int eventMask = entry.getEventMask();
+            final SelectionKey key = entry.getSelectionKey();
+            final int eventMask = entry.getEventMask();
             if (key.isValid()) {
                 key.interestOps(eventMask);
             }
@@ -461,9 +462,7 @@ public abstract class AbstractIOReactor implements IOReactor {
 
     private boolean queueInterestOps(final InterestOpEntry entry) {
         // validity checks
-        if (!this.interestOpsQueueing) {
-            throw new IllegalStateException("Interest ops queueing not enabled");
-        }
+        Asserts.check(this.interestOpsQueueing, "Interest ops queueing not enabled");
         if (entry == null) {
             return false;
         }
@@ -483,10 +482,10 @@ public abstract class AbstractIOReactor implements IOReactor {
      * @param key the selection key.
      * @param now current time as long value.
      */
-    protected void timeoutCheck(final SelectionKey key, long now) {
-        IOSessionImpl session = (IOSessionImpl) key.attachment();
+    protected void timeoutCheck(final SelectionKey key, final long now) {
+        final IOSessionImpl session = (IOSessionImpl) key.attachment();
         if (session != null) {
-            int timeout = session.getSocketTimeout();
+            final int timeout = session.getSocketTimeout();
             if (timeout > 0) {
                 if (session.getLastAccessTime() + timeout < now) {
                     sessionTimedOut(session);
@@ -500,8 +499,7 @@ public abstract class AbstractIOReactor implements IOReactor {
      */
     protected void closeSessions() {
         synchronized (this.sessions) {
-            for (Iterator<IOSession> it = this.sessions.iterator(); it.hasNext(); ) {
-                IOSession session = it.next();
+            for (final IOSession session : this.sessions) {
                 session.close();
             }
         }
@@ -515,14 +513,14 @@ public abstract class AbstractIOReactor implements IOReactor {
     protected void closeNewChannels() throws IOReactorException {
         ChannelEntry entry;
         while ((entry = this.newChannels.poll()) != null) {
-            SessionRequestImpl sessionRequest = entry.getSessionRequest();
+            final SessionRequestImpl sessionRequest = entry.getSessionRequest();
             if (sessionRequest != null) {
                 sessionRequest.cancel();
             }
-            SocketChannel channel = entry.getChannel();
+            final SocketChannel channel = entry.getChannel();
             try {
                 channel.close();
-            } catch (IOException ignore) {
+            } catch (final IOException ignore) {
             }
         }
     }
@@ -534,16 +532,15 @@ public abstract class AbstractIOReactor implements IOReactor {
      */
     protected void closeActiveChannels() throws IOReactorException {
         try {
-            Set<SelectionKey> keys = this.selector.keys();
-            for (Iterator<SelectionKey> it = keys.iterator(); it.hasNext(); ) {
-                SelectionKey key = it.next();
-                IOSession session = getSession(key);
+            final Set<SelectionKey> keys = this.selector.keys();
+            for (final SelectionKey key : keys) {
+                final IOSession session = getSession(key);
                 if (session != null) {
                     session.close();
                 }
             }
             this.selector.close();
-        } catch (IOException ignore) {
+        } catch (final IOException ignore) {
         }
     }
 
@@ -585,9 +582,9 @@ public abstract class AbstractIOReactor implements IOReactor {
      * @param timeout the maximum wait time.
      * @throws InterruptedException if interrupted.
      */
-    public void awaitShutdown(long timeout) throws InterruptedException {
+    public void awaitShutdown(final long timeout) throws InterruptedException {
         synchronized (this.statusMutex) {
-            long deadline = System.currentTimeMillis() + timeout;
+            final long deadline = System.currentTimeMillis() + timeout;
             long remaining = timeout;
             while (this.status != IOReactorStatus.SHUT_DOWN) {
                 this.statusMutex.wait(remaining);
@@ -601,12 +598,12 @@ public abstract class AbstractIOReactor implements IOReactor {
         }
     }
 
-    public void shutdown(long gracePeriod) throws IOReactorException {
+    public void shutdown(final long gracePeriod) throws IOReactorException {
         if (this.status != IOReactorStatus.INACTIVE) {
             gracefulShutdown();
             try {
                 awaitShutdown(gracePeriod);
-            } catch (InterruptedException ignore) {
+            } catch (final InterruptedException ignore) {
             }
         }
         if (this.status != IOReactorStatus.SHUT_DOWN) {
