@@ -38,19 +38,21 @@ import java.nio.channels.SelectionKey;
 import java.nio.channels.Selector;
 import java.util.ArrayList;
 import java.util.Date;
-import java.util.Iterator;
 import java.util.List;
-import java.util.Set;
 import java.util.concurrent.ThreadFactory;
 
-import org.apache.http.nio.params.NIOReactorParams;
+import org.apache.http.annotation.ThreadSafe;
+import org.apache.http.nio.params.NIOReactorPNames;
 import org.apache.http.nio.reactor.IOEventDispatch;
 import org.apache.http.nio.reactor.IOReactor;
 import org.apache.http.nio.reactor.IOReactorException;
 import org.apache.http.nio.reactor.IOReactorExceptionHandler;
 import org.apache.http.nio.reactor.IOReactorStatus;
-import org.apache.http.params.HttpConnectionParams;
+import org.apache.http.params.BasicHttpParams;
+import org.apache.http.params.CoreConnectionPNames;
 import org.apache.http.params.HttpParams;
+import org.apache.http.util.Args;
+import org.apache.http.util.Asserts;
 
 /**
  * Generic implementation of {@link IOReactor} that can run multiple
@@ -89,25 +91,21 @@ import org.apache.http.params.HttpParams;
  * can obtain the audit log using {@link #getAuditLog()}, examine exceptions
  * thrown by the I/O reactor prior and in the course of the reactor shutdown
  * and decide whether it is safe to restart the I/O reactor.
- * <p>
- * The following parameters can be used to customize the behavior of this
- * class:
- * <ul>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#TCP_NODELAY}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#SO_TIMEOUT}</li>
- *  <li>{@link org.apache.http.params.CoreConnectionPNames#SO_LINGER}</li>
- *  <li>{@link org.apache.http.nio.params.NIOReactorPNames#SELECT_INTERVAL}</li>
- *  <li>{@link org.apache.http.nio.params.NIOReactorPNames#GRACE_PERIOD}</li>
- *  <li>{@link org.apache.http.nio.params.NIOReactorPNames#INTEREST_OPS_QUEUEING}</li>
- * </ul>
  *
  * @since 4.0
  */
+@SuppressWarnings("deprecation")
+@ThreadSafe // public methods only
 public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     protected volatile IOReactorStatus status;
 
+    /**
+     * @deprecated (4.2)
+     */
+    @Deprecated
     protected final HttpParams params;
+    protected final IOReactorConfig config;
     protected final Selector selector;
     protected final long selectTimeout;
     protected final boolean interestOpsQueueing;
@@ -124,6 +122,71 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
     private int currentWorker = 0;
 
+    /**
+     * Creates an instance of AbstractMultiworkerIOReactor with the given configuration.
+     *
+     * @param config I/O reactor configuration.
+     * @param threadFactory the factory to create threads.
+     *   Can be <code>null</code>.
+     * @throws IOReactorException in case if a non-recoverable I/O error.
+     *
+     * @since 4.2
+     */
+    public AbstractMultiworkerIOReactor(
+            final IOReactorConfig config,
+            final ThreadFactory threadFactory) throws IOReactorException {
+        super();
+        this.config = config != null ? config : IOReactorConfig.DEFAULT;
+        this.params = new BasicHttpParams();
+        try {
+            this.selector = Selector.open();
+        } catch (final IOException ex) {
+            throw new IOReactorException("Failure opening selector", ex);
+        }
+        this.selectTimeout = this.config.getSelectInterval();
+        this.interestOpsQueueing = this.config.isInterestOpQueued();
+        this.statusLock = new Object();
+        if (threadFactory != null) {
+            this.threadFactory = threadFactory;
+        } else {
+            this.threadFactory = new DefaultThreadFactory();
+        }
+        this.workerCount = this.config.getIoThreadCount();
+        this.dispatchers = new BaseIOReactor[workerCount];
+        this.workers = new Worker[workerCount];
+        this.threads = new Thread[workerCount];
+        this.status = IOReactorStatus.INACTIVE;
+    }
+
+    /**
+     * Creates an instance of AbstractMultiworkerIOReactor with default configuration.
+     *
+     * @throws IOReactorException in case if a non-recoverable I/O error.
+     *
+     * @since 4.2
+     */
+    public AbstractMultiworkerIOReactor() throws IOReactorException {
+        this(null, null);
+    }
+
+    @Deprecated
+    static IOReactorConfig convert(final int workerCount, final HttpParams params) {
+        Args.notNull(params, "HTTP parameters");
+        return IOReactorConfig.custom()
+            .setSelectInterval(params.getLongParameter(NIOReactorPNames.SELECT_INTERVAL, 1000))
+            .setShutdownGracePeriod(params.getLongParameter(NIOReactorPNames.GRACE_PERIOD, 500))
+            .setInterestOpQueued(params.getBooleanParameter(NIOReactorPNames.SELECT_INTERVAL, false))
+            .setIoThreadCount(workerCount)
+            .setSoTimeout(params.getIntParameter(CoreConnectionPNames.SO_TIMEOUT, 0))
+            .setConnectTimeout(params.getIntParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, 0))
+            .setSoTimeout(params.getIntParameter(CoreConnectionPNames.SO_TIMEOUT, 0))
+            .setSoReuseAddress(params.getBooleanParameter(CoreConnectionPNames.SO_REUSEADDR, false))
+            .setSoKeepAlive(params.getBooleanParameter(CoreConnectionPNames.SO_KEEPALIVE, false))
+            .setSoLinger(params.getIntParameter(CoreConnectionPNames.SO_LINGER, -1))
+            .setTcpNoDelay(params.getBooleanParameter(CoreConnectionPNames.TCP_NODELAY, true))
+            .build();
+    }
+
     /**
      * Creates an instance of AbstractMultiworkerIOReactor.
      *
@@ -132,37 +195,15 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      *   Can be <code>null</code>.
      * @param params HTTP parameters.
      * @throws IOReactorException in case if a non-recoverable I/O error.
+     *
+     * @deprecated (4.2) use {@link AbstractMultiworkerIOReactor#AbstractMultiworkerIOReactor(IOReactorConfig, ThreadFactory)}
      */
+    @Deprecated
     public AbstractMultiworkerIOReactor(
-            int workerCount,
+            final int workerCount,
             final ThreadFactory threadFactory,
             final HttpParams params) throws IOReactorException {
-        super();
-        if (workerCount <= 0) {
-            throw new IllegalArgumentException("Worker count may not be negative or zero");
-        }
-        if (params == null) {
-            throw new IllegalArgumentException("HTTP parameters may not be null");
-        }
-        try {
-            this.selector = Selector.open();
-        } catch (IOException ex) {
-            throw new IOReactorException("Failure opening selector", ex);
-        }
-        this.params = params;
-        this.selectTimeout = NIOReactorParams.getSelectInterval(params);
-        this.interestOpsQueueing = NIOReactorParams.getInterestOpsQueueing(params);
-        this.statusLock = new Object();
-        this.workerCount = workerCount;
-        if (threadFactory != null) {
-            this.threadFactory = threadFactory;
-        } else {
-            this.threadFactory = new DefaultThreadFactory();
-        }
-        this.dispatchers = new BaseIOReactor[workerCount];
-        this.workers = new Worker[workerCount];
-        this.threads = new Thread[workerCount];
-        this.status = IOReactorStatus.INACTIVE;
+        this(convert(workerCount, params), threadFactory);
     }
 
     public IOReactorStatus getStatus() {
@@ -191,17 +232,14 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      * @param timestamp the time stamp of the exception. Can be
      * <code>null</code> in which case the current date / time will be used.
      */
-    protected synchronized void addExceptionEvent(final Throwable ex, Date timestamp) {
+    protected synchronized void addExceptionEvent(final Throwable ex, final Date timestamp) {
         if (ex == null) {
             return;
         }
-        if (timestamp == null) {
-            timestamp = new Date();
-        }
         if (this.auditLog == null) {
             this.auditLog = new ArrayList<ExceptionEvent>();
         }
-        this.auditLog.add(new ExceptionEvent(ex, timestamp));
+        this.auditLog.add(new ExceptionEvent(ex, timestamp != null ? timestamp : new Date()));
     }
 
     /**
@@ -265,27 +303,24 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      */
     public void execute(
             final IOEventDispatch eventDispatch) throws InterruptedIOException, IOReactorException {
-        if (eventDispatch == null) {
-            throw new IllegalArgumentException("Event dispatcher may not be null");
-        }
+        Args.notNull(eventDispatch, "Event dispatcher");
         synchronized (this.statusLock) {
             if (this.status.compareTo(IOReactorStatus.SHUTDOWN_REQUEST) >= 0) {
                 this.status = IOReactorStatus.SHUT_DOWN;
                 this.statusLock.notifyAll();
                 return;
             }
-            if (this.status.compareTo(IOReactorStatus.INACTIVE) != 0) {
-                throw new IllegalStateException("Illegal state: " + this.status);
-            }
+            Asserts.check(this.status.compareTo(IOReactorStatus.INACTIVE) == 0,
+                    "Illegal state %s", this.status);
             this.status = IOReactorStatus.ACTIVE;
             // Start I/O dispatchers
             for (int i = 0; i < this.dispatchers.length; i++) {
-                BaseIOReactor dispatcher = new BaseIOReactor(this.selectTimeout, this.interestOpsQueueing);
+                final BaseIOReactor dispatcher = new BaseIOReactor(this.selectTimeout, this.interestOpsQueueing);
                 dispatcher.setExceptionHandler(exceptionHandler);
                 this.dispatchers[i] = dispatcher;
             }
             for (int i = 0; i < this.workerCount; i++) {
-                BaseIOReactor dispatcher = this.dispatchers[i];
+                final BaseIOReactor dispatcher = this.dispatchers[i];
                 this.workers[i] = new Worker(dispatcher, eventDispatch);
                 this.threads[i] = this.threadFactory.newThread(this.workers[i]);
             }
@@ -300,12 +335,12 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
             }
 
             for (;;) {
-                int readyCount;
+                final int readyCount;
                 try {
                     readyCount = this.selector.select(this.selectTimeout);
-                } catch (InterruptedIOException ex) {
+                } catch (final InterruptedIOException ex) {
                     throw ex;
-                } catch (IOException ex) {
+                } catch (final IOException ex) {
                     throw new IOReactorException("Unexpected selector failure", ex);
                 }
 
@@ -315,8 +350,8 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
                 // Verify I/O dispatchers
                 for (int i = 0; i < this.workerCount; i++) {
-                    Worker worker = this.workers[i];
-                    Exception ex = worker.getException();
+                    final Worker worker = this.workers[i];
+                    final Exception ex = worker.getException();
                     if (ex != null) {
                         throw new IOReactorException(
                                 "I/O dispatch worker terminated abnormally", ex);
@@ -328,9 +363,9 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
                 }
             }
 
-        } catch (ClosedSelectorException ex) {
+        } catch (final ClosedSelectorException ex) {
             addExceptionEvent(ex);
-        } catch (IOReactorException ex) {
+        } catch (final IOReactorException ex) {
             if (ex.getCause() != null) {
                 addExceptionEvent(ex.getCause());
             }
@@ -363,7 +398,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         }
         try {
             cancelRequests();
-        } catch (IOReactorException ex) {
+        } catch (final IOReactorException ex) {
             if (ex.getCause() != null) {
                 addExceptionEvent(ex.getCause());
             }
@@ -372,46 +407,44 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
 
         // Close out all channels
         if (this.selector.isOpen()) {
-            Set<SelectionKey> keys = this.selector.keys();
-            for (Iterator<SelectionKey> it = keys.iterator(); it.hasNext(); ) {
+            for (final SelectionKey key : this.selector.keys()) {
                 try {
-                    SelectionKey key = it.next();
-                    Channel channel = key.channel();
+                    final Channel channel = key.channel();
                     if (channel != null) {
                         channel.close();
                     }
-                } catch (IOException ex) {
+                } catch (final IOException ex) {
                     addExceptionEvent(ex);
                 }
             }
             // Stop dispatching I/O events
             try {
                 this.selector.close();
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 addExceptionEvent(ex);
             }
         }
 
         // Attempt to shut down I/O dispatchers gracefully
         for (int i = 0; i < this.workerCount; i++) {
-            BaseIOReactor dispatcher = this.dispatchers[i];
+            final BaseIOReactor dispatcher = this.dispatchers[i];
             dispatcher.gracefulShutdown();
         }
 
-        long gracePeriod = NIOReactorParams.getGracePeriod(this.params);
+        final long gracePeriod = this.config.getShutdownGracePeriod();
 
         try {
             // Force shut down I/O dispatchers if they fail to terminate
             // in time
             for (int i = 0; i < this.workerCount; i++) {
-                BaseIOReactor dispatcher = this.dispatchers[i];
+                final BaseIOReactor dispatcher = this.dispatchers[i];
                 if (dispatcher.getStatus() != IOReactorStatus.INACTIVE) {
                     dispatcher.awaitShutdown(gracePeriod);
                 }
                 if (dispatcher.getStatus() != IOReactorStatus.SHUT_DOWN) {
                     try {
                         dispatcher.hardShutdown();
-                    } catch (IOReactorException ex) {
+                    } catch (final IOReactorException ex) {
                         if (ex.getCause() != null) {
                             addExceptionEvent(ex.getCause());
                         }
@@ -420,12 +453,12 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
             }
             // Join worker threads
             for (int i = 0; i < this.workerCount; i++) {
-                Thread t = this.threads[i];
+                final Thread t = this.threads[i];
                 if (t != null) {
                     t.join(gracePeriod);
                 }
             }
-        } catch (InterruptedException ex) {
+        } catch (final InterruptedException ex) {
             throw new InterruptedIOException(ex.getMessage());
         }
     }
@@ -437,7 +470,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      */
     protected void addChannel(final ChannelEntry entry) {
         // Distribute new channels among the workers
-        int i = Math.abs(this.currentWorker++ % this.workerCount);
+        final int i = Math.abs(this.currentWorker++ % this.workerCount);
         this.dispatchers[i].addChannel(entry);
     }
 
@@ -450,7 +483,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      * @throws ClosedChannelException if the channel has been already closed.
      */
     protected SelectionKey registerChannel(
-            final SelectableChannel channel, int ops) throws ClosedChannelException {
+            final SelectableChannel channel, final int ops) throws ClosedChannelException {
         return channel.register(this.selector, ops);
     }
 
@@ -461,9 +494,18 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      * @throws IOException in case of an I/O error.
      */
     protected void prepareSocket(final Socket socket) throws IOException {
-        socket.setTcpNoDelay(HttpConnectionParams.getTcpNoDelay(this.params));
-        socket.setSoTimeout(HttpConnectionParams.getSoTimeout(this.params));
-        int linger = HttpConnectionParams.getLinger(this.params);
+        socket.setTcpNoDelay(this.config.isTcpNoDelay());
+        socket.setKeepAlive(this.config.isSoKeepalive());
+        if (this.config.getSoTimeout() > 0) {
+            socket.setSoTimeout(this.config.getSoTimeout());
+        }
+        if (this.config.getSndBufSize() > 0) {
+            socket.setSendBufferSize(this.config.getSndBufSize());
+        }
+        if (this.config.getRcvBufSize() > 0) {
+            socket.setReceiveBufferSize(this.config.getRcvBufSize());
+        }
+        final int linger = this.config.getSoLinger();
         if (linger >= 0) {
             socket.setSoLinger(linger > 0, linger);
         }
@@ -478,9 +520,9 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
      * @param timeout the maximum wait time.
      * @throws InterruptedException if interrupted.
      */
-    protected void awaitShutdown(long timeout) throws InterruptedException {
+    protected void awaitShutdown(final long timeout) throws InterruptedException {
         synchronized (this.statusLock) {
-            long deadline = System.currentTimeMillis() + timeout;
+            final long deadline = System.currentTimeMillis() + timeout;
             long remaining = timeout;
             while (this.status != IOReactorStatus.SHUT_DOWN) {
                 this.statusLock.wait(remaining);
@@ -498,7 +540,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         shutdown(2000);
     }
 
-    public void shutdown(long waitMs) throws IOException {
+    public void shutdown(final long waitMs) throws IOException {
         synchronized (this.statusLock) {
             if (this.status.compareTo(IOReactorStatus.ACTIVE) > 0) {
                 return;
@@ -513,14 +555,14 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         this.selector.wakeup();
         try {
             awaitShutdown(waitMs);
-        } catch (InterruptedException ignore) {
+        } catch (final InterruptedException ignore) {
         }
     }
 
     static void closeChannel(final Channel channel) {
         try {
             channel.close();
-        } catch (IOException ignore) {
+        } catch (final IOException ignore) {
         }
     }
 
@@ -540,7 +582,7 @@ public abstract class AbstractMultiworkerIOReactor implements IOReactor {
         public void run() {
             try {
                 this.dispatcher.execute(this.eventDispatch);
-            } catch (Exception ex) {
+            } catch (final Exception ex) {
                 this.exception = ex;
             }
         }
