@@ -29,11 +29,14 @@ package org.apache.http.impl.nio.reactor;
 
 import java.util.Date;
 
+import org.apache.http.annotation.Immutable;
+
 /**
  * A {@link Throwable} instance along with a time stamp.
  *
  * @since 4.0
  */
+@Immutable
 public class ExceptionEvent {
 
     private final Throwable ex;
@@ -63,7 +66,7 @@ public class ExceptionEvent {
 
     @Override
     public String toString() {
-        StringBuilder buffer = new StringBuilder();
+        final StringBuilder buffer = new StringBuilder();
         buffer.append(new Date(this.time));
         buffer.append(" ");
         buffer.append(this.ex);
