@@ -32,15 +32,18 @@ import java.net.SocketAddress;
 import java.nio.channels.Channel;
 import java.nio.channels.SelectionKey;
 
+import org.apache.http.annotation.ThreadSafe;
 import org.apache.http.nio.reactor.IOSession;
 import org.apache.http.nio.reactor.SessionRequest;
 import org.apache.http.nio.reactor.SessionRequestCallback;
+import org.apache.http.util.Args;
 
 /**
  * Default implementation of {@link SessionRequest}.
  *
  * @since 4.0
  */
+@ThreadSafe
 public class SessionRequestImpl implements SessionRequest {
 
     private volatile boolean completed;
@@ -61,9 +64,7 @@ public class SessionRequestImpl implements SessionRequest {
             final Object attachment,
             final SessionRequestCallback callback) {
         super();
-        if (remoteAddress == null) {
-            throw new IllegalArgumentException("Remote address may not be null");
-        }
+        Args.notNull(remoteAddress, "Remote address");
         this.remoteAddress = remoteAddress;
         this.localAddress = localAddress;
         this.attachment = attachment;
@@ -115,9 +116,7 @@ public class SessionRequestImpl implements SessionRequest {
     }
 
     public void completed(final IOSession session) {
-        if (session == null) {
-            throw new IllegalArgumentException("Session may not be null");
-        }
+        Args.notNull(session, "Session");
         if (this.completed) {
             return;
         }
@@ -139,14 +138,14 @@ public class SessionRequestImpl implements SessionRequest {
             return;
         }
         this.completed = true;
-        SelectionKey key = this.key;
+        final SelectionKey key = this.key;
         if (key != null) {
             key.cancel();
-            Channel channel = key.channel();
+            final Channel channel = key.channel();
             if (channel.isOpen()) {
                 try {
                     channel.close();
-                } catch (IOException ignore) {}
+                } catch (final IOException ignore) {}
             }
         }
         synchronized (this) {
@@ -163,14 +162,14 @@ public class SessionRequestImpl implements SessionRequest {
             return;
         }
         this.completed = true;
-        SelectionKey key = this.key;
+        final SelectionKey key = this.key;
         if (key != null) {
             key.cancel();
-            Channel channel = key.channel();
+            final Channel channel = key.channel();
             if (channel.isOpen()) {
                 try {
                     channel.close();
-                } catch (IOException ignore) {}
+                } catch (final IOException ignore) {}
             }
         }
         synchronized (this) {
@@ -184,10 +183,10 @@ public class SessionRequestImpl implements SessionRequest {
         return this.connectTimeout;
     }
 
-    public void setConnectTimeout(int timeout) {
+    public void setConnectTimeout(final int timeout) {
         if (this.connectTimeout != timeout) {
             this.connectTimeout = timeout;
-            SelectionKey key = this.key;
+            final SelectionKey key = this.key;
             if (key != null) {
                 key.selector().wakeup();
             }
@@ -199,14 +198,14 @@ public class SessionRequestImpl implements SessionRequest {
             return;
         }
         this.completed = true;
-        SelectionKey key = this.key;
+        final SelectionKey key = this.key;
         if (key != null) {
             key.cancel();
-            Channel channel = key.channel();
+            final Channel channel = key.channel();
             if (channel.isOpen()) {
                 try {
                     channel.close();
-                } catch (IOException ignore) {}
+                } catch (final IOException ignore) {}
             }
         }
         synchronized (this) {
