@@ -31,8 +31,6 @@ import java.io.IOException;
 import java.nio.ByteBuffer;
 import java.nio.channels.WritableByteChannel;
 
-import org.apache.http.nio.ContentEncoder;
-
 /**
  * A {@link WritableByteChannel} that delegates to a {@link ContentEncoder}.
  * Attempts to close this channel are ignored, and {@link #isOpen} always
@@ -44,11 +42,11 @@ public class ContentEncoderChannel implements WritableByteChannel {
 
     private final ContentEncoder contentEncoder;
 
-    public ContentEncoderChannel(ContentEncoder contentEncoder) {
+    public ContentEncoderChannel(final ContentEncoder contentEncoder) {
         this.contentEncoder = contentEncoder;
     }
 
-    public int write(ByteBuffer src) throws IOException {
+    public int write(final ByteBuffer src) throws IOException {
         return contentEncoder.write(src);
     }
 
