@@ -35,7 +35,10 @@ import org.apache.http.HttpException;
  * Abstract client-side HTTP protocol handler.
  *
  * @since 4.0
+ *
+ * @deprecated (4.2) use {@link NHttpClientEventHandler}
  */
+@Deprecated
 public interface NHttpClientHandler {
 
     /**
