@@ -33,42 +33,113 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.nio.ByteBuffer;
 
+import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.entity.AbstractHttpEntity;
+import org.apache.http.entity.ContentType;
 import org.apache.http.nio.ContentEncoder;
 import org.apache.http.nio.IOControl;
-import org.apache.http.nio.protocol.AsyncNHttpServiceHandler;
+import org.apache.http.util.Args;
 
 /**
  * A simple self contained, repeatable non-blocking entity that retrieves
  * its content from a byte array.
  *
- * @see AsyncNHttpServiceHandler
  * @since 4.0
  */
-public class NByteArrayEntity
-    extends AbstractHttpEntity implements ProducingNHttpEntity {
+@SuppressWarnings("deprecation")
+@NotThreadSafe
+public class NByteArrayEntity extends AbstractHttpEntity
+                              implements HttpAsyncContentProducer, ProducingNHttpEntity {
 
+    private final byte[] b;
+    private final int off, len;
+    private final ByteBuffer buf;
+    /**
+     * @deprecated (4.2)
+     */
+    @Deprecated
     protected final byte[] content;
+    /**
+     * @deprecated (4.2)
+     */
+    @Deprecated
     protected final ByteBuffer buffer;
 
-    public NByteArrayEntity(final byte[] b) {
+    /**
+     * @since 4.2
+     */
+    public NByteArrayEntity(final byte[] b, final ContentType contentType) {
+        super();
+        Args.notNull(b, "Source byte array");
+        this.b = b;
+        this.off = 0;
+        this.len = b.length;
+        this.buf = ByteBuffer.wrap(b);
         this.content = b;
-        this.buffer = ByteBuffer.wrap(b);
+        this.buffer = this.buf;
+        if (contentType != null) {
+            setContentType(contentType.toString());
+        }
     }
 
+    /**
+     * @since 4.2
+     */
+    public NByteArrayEntity(final byte[] b, final int off, final int len, final ContentType contentType) {
+        super();
+        Args.notNull(b, "Source byte array");
+        if ((off < 0) || (off > b.length) || (len < 0) ||
+                ((off + len) < 0) || ((off + len) > b.length)) {
+            throw new IndexOutOfBoundsException("off: " + off + " len: " + len + " b.length: " + b.length);
+        }
+        this.b = b;
+        this.off = off;
+        this.len = len;
+        this.buf = ByteBuffer.wrap(b, off, len);
+        this.content = b;
+        this.buffer = this.buf;
+        if (contentType != null) {
+            setContentType(contentType.toString());
+        }
+    }
+
+    public NByteArrayEntity(final byte[] b) {
+        this(b, null);
+    }
+
+    public NByteArrayEntity(final byte[] b, final int off, final int len) {
+        this(b, off, len, null);
+    }
+
+    /**
+     * {@inheritDoc}
+     *
+     * @since 4.2
+     */
+    public void close() {
+        this.buf.rewind();
+    }
+
+    /**
+     * {@inheritDoc}
+     *
+     * @deprecated (4.2) use {@link #close()}
+     */
+    @Deprecated
     public void finish() {
-        buffer.rewind();
+        close();
     }
 
-    public void produceContent(ContentEncoder encoder, IOControl ioctrl)
+    public void produceContent(final ContentEncoder encoder, final IOControl ioctrl)
             throws IOException {
-        encoder.write(buffer);
-        if(!buffer.hasRemaining())
+        encoder.write(this.buf);
+        if(!this.buf.hasRemaining()) {
             encoder.complete();
+        }
     }
 
     public long getContentLength() {
-        return buffer.limit();
+        return this.len;
     }
 
     public boolean isRepeatable() {
@@ -80,14 +151,12 @@ public class NByteArrayEntity
     }
 
     public InputStream getContent() {
-        return new ByteArrayInputStream(content);
+        return new ByteArrayInputStream(this.b, this.off, this.len);
     }
 
     public void writeTo(final OutputStream outstream) throws IOException {
-        if (outstream == null) {
-            throw new IllegalArgumentException("Output stream may not be null");
-        }
-        outstream.write(content);
+        Args.notNull(outstream, "Output stream");
+        outstream.write(this.b, this.off, this.len);
         outstream.flush();
     }
 
