@@ -655,7 +655,18 @@ public abstract class AbstractNIOConnPool<T, C, E extends PoolEntry<T, C>>
     protected void enumAvailable(final PoolEntryCallback<T, C> callback) {
         this.lock.lock();
         try {
-            enumEntries(this.available.iterator(), callback);
+            final Iterator<E> it = this.available.iterator();
+            while (it.hasNext()) {
+                final E entry = it.next();
+                callback.process(entry);
+                if (entry.isClosed()) {
+                    final RouteSpecificPool<T, C, E> pool = getPool(entry.getRoute());
+                    pool.remove(entry);
+                    it.remove();
+                }
+            }
+            processPendingRequests();
+            purgePoolMap();
         } finally {
             this.lock.unlock();
         }
@@ -669,23 +680,41 @@ public abstract class AbstractNIOConnPool<T, C, E extends PoolEntry<T, C>>
     protected void enumLeased(final PoolEntryCallback<T, C> callback) {
         this.lock.lock();
         try {
-            enumEntries(this.leased.iterator(), callback);
+            final Iterator<E> it = this.leased.iterator();
+            while (it.hasNext()) {
+                final E entry = it.next();
+                callback.process(entry);
+            }
+            processPendingRequests();
         } finally {
             this.lock.unlock();
         }
     }
 
+    /**
+     * Use {@link #enumLeased(org.apache.http.pool.PoolEntryCallback)}
+     *  or {@link #enumAvailable(org.apache.http.pool.PoolEntryCallback)} instead.
+     *
+     * @deprecated (4.3.2)
+     */
+    @Deprecated
     protected void enumEntries(final Iterator<E> it, final PoolEntryCallback<T, C> callback) {
         while (it.hasNext()) {
             final E entry = it.next();
             callback.process(entry);
-            if (entry.isClosed()) {
-                final RouteSpecificPool<T, C, E> pool = getPool(entry.getRoute());
-                pool.remove(entry);
+        }
+        processPendingRequests();
+    }
+
+    private void purgePoolMap() {
+        final Iterator<Map.Entry<T, RouteSpecificPool<T, C, E>>> it = this.routeToPool.entrySet().iterator();
+        while (it.hasNext()) {
+            final Map.Entry<T, RouteSpecificPool<T, C, E>> entry = it.next();
+            final RouteSpecificPool<T, C, E> pool = entry.getValue();
+            if (pool.getAllocatedCount() == 0) {
                 it.remove();
             }
         }
-        processPendingRequests();
     }
 
     public void closeIdle(final long idletime, final TimeUnit tunit) {
