@@ -29,7 +29,9 @@ package org.apache.http.nio.util;
 
 import java.nio.ByteBuffer;
 
+import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.io.BufferInfo;
+import org.apache.http.util.Args;
 
 /**
  * A buffer that expand its capacity on demand using {@link ByteBufferAllocator}
@@ -41,14 +43,16 @@ import org.apache.http.io.BufferInfo;
  * @since 4.0
  */
 @SuppressWarnings("deprecation")
+@NotThreadSafe
 public class ExpandableBuffer implements BufferInfo, org.apache.http.nio.util.BufferInfo {
 
     public final static int INPUT_MODE = 0;
     public final static int OUTPUT_MODE = 1;
 
+    private final ByteBufferAllocator allocator;
+
     private int mode;
     protected ByteBuffer buffer = null;
-    private final ByteBufferAllocator allocator;
 
     /**
      * Allocates buffer of the given size using the given allocator.
@@ -56,11 +60,9 @@ public class ExpandableBuffer implements BufferInfo, org.apache.http.nio.util.Bu
      * @param buffersize the buffer size.
      * @param allocator allocator to be used to allocate {@link ByteBuffer}s.
      */
-    public ExpandableBuffer(int buffersize, final ByteBufferAllocator allocator) {
+    public ExpandableBuffer(final int buffersize, final ByteBufferAllocator allocator) {
         super();
-        if (allocator == null) {
-            throw new IllegalArgumentException("ByteBuffer allocator may not be null");
-        }
+        Args.notNull(allocator, "ByteBuffer allocator");
         this.allocator = allocator;
         this.buffer = allocator.allocate(buffersize);
         this.mode = INPUT_MODE;
@@ -103,8 +105,8 @@ public class ExpandableBuffer implements BufferInfo, org.apache.http.nio.util.Bu
         }
     }
 
-    private void expandCapacity(int capacity) {
-        ByteBuffer oldbuffer = this.buffer;
+    private void expandCapacity(final int capacity) {
+        final ByteBuffer oldbuffer = this.buffer;
         this.buffer = allocator.allocate(capacity);
         oldbuffer.flip();
         this.buffer.put(oldbuffer);
@@ -123,10 +125,8 @@ public class ExpandableBuffer implements BufferInfo, org.apache.http.nio.util.Bu
 
     /**
      * Ensures the buffer can accommodate the required capacity.
-     *
-     * @param requiredCapacity
      */
-    protected void ensureCapacity(int requiredCapacity) {
+    protected void ensureCapacity(final int requiredCapacity) {
         if (requiredCapacity > this.buffer.capacity()) {
             expandCapacity(requiredCapacity);
         }
@@ -182,10 +182,9 @@ public class ExpandableBuffer implements BufferInfo, org.apache.http.nio.util.Bu
 
     @Override
     public String toString() {
-        StringBuffer sb = new StringBuffer();
+        final StringBuilder sb = new StringBuilder();
         sb.append("[mode=");
-        int mode = getMode();
-        if (mode == INPUT_MODE) {
+        if (getMode() == INPUT_MODE) {
             sb.append("in");
         } else {
             sb.append("out");
