@@ -29,15 +29,24 @@ package org.apache.http.nio.util;
 
 import java.nio.ByteBuffer;
 
+import org.apache.http.annotation.Immutable;
+
 /**
  * Allocates {@link ByteBuffer} instances using
  * {@link ByteBuffer#allocate(int)}.
  *
  * @since 4.0
  */
+@Immutable
 public class HeapByteBufferAllocator implements ByteBufferAllocator {
 
-    public ByteBuffer allocate(int size) {
+    public static final HeapByteBufferAllocator INSTANCE = new HeapByteBufferAllocator();
+
+    public HeapByteBufferAllocator() {
+        super();
+    }
+
+    public ByteBuffer allocate(final int size) {
         return ByteBuffer.allocate(size);
     }
 
