@@ -28,6 +28,7 @@ package org.apache.http.nio.util;
 
 import java.io.IOException;
 
+import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.nio.ContentDecoder;
 
 /**
@@ -37,14 +38,22 @@ import org.apache.http.nio.ContentDecoder;
  *
  * @since 4.0
  */
+@NotThreadSafe
 public class SimpleInputBuffer extends ExpandableBuffer implements ContentInputBuffer {
 
     private boolean endOfStream = false;
 
-    public SimpleInputBuffer(int buffersize, final ByteBufferAllocator allocator) {
+    public SimpleInputBuffer(final int buffersize, final ByteBufferAllocator allocator) {
         super(buffersize, allocator);
     }
 
+    /**
+     * @since 4.3
+     */
+    public SimpleInputBuffer(final int buffersize) {
+        this(buffersize, HeapByteBufferAllocator.INSTANCE);
+    }
+
     public void reset() {
         this.endOfStream = false;
         super.clear();
@@ -79,10 +88,11 @@ public class SimpleInputBuffer extends ExpandableBuffer implements ContentInputB
         if (isEndOfStream()) {
             return -1;
         }
+        setOutputMode();
         return this.buffer.get() & 0xff;
     }
 
-    public int read(final byte[] b, int off, int len) throws IOException {
+    public int read(final byte[] b, final int off, final int len) throws IOException {
         if (isEndOfStream()) {
             return -1;
         }
