@@ -28,6 +28,7 @@ package org.apache.http.nio.util;
 
 import java.io.IOException;
 
+import org.apache.http.annotation.NotThreadSafe;
 import org.apache.http.nio.ContentEncoder;
 
 /**
@@ -37,25 +38,33 @@ import org.apache.http.nio.ContentEncoder;
  *
  * @since 4.0
  */
+@NotThreadSafe
 public class SimpleOutputBuffer extends ExpandableBuffer implements ContentOutputBuffer {
 
     private boolean endOfStream;
 
-    public SimpleOutputBuffer(int buffersize, final ByteBufferAllocator allocator) {
+    public SimpleOutputBuffer(final int buffersize, final ByteBufferAllocator allocator) {
         super(buffersize, allocator);
         this.endOfStream = false;
     }
 
+    /**
+     * @since 4.3
+     */
+    public SimpleOutputBuffer(final int buffersize) {
+        this(buffersize, HeapByteBufferAllocator.INSTANCE);
+    }
+
     public int produceContent(final ContentEncoder encoder) throws IOException {
         setOutputMode();
-        int bytesWritten = encoder.write(this.buffer);
+        final int bytesWritten = encoder.write(this.buffer);
         if (!hasData() && this.endOfStream) {
             encoder.complete();
         }
         return bytesWritten;
     }
 
-    public void write(final byte[] b, int off, int len) throws IOException {
+    public void write(final byte[] b, final int off, final int len) throws IOException {
         if (b == null) {
             return;
         }
@@ -77,7 +86,7 @@ public class SimpleOutputBuffer extends ExpandableBuffer implements ContentOutpu
         write(b, 0, b.length);
     }
 
-    public void write(int b) throws IOException {
+    public void write(final int b) throws IOException {
         if (this.endOfStream) {
             return;
         }
