@@ -34,68 +34,54 @@ import java.util.Set;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 
-import junit.framework.TestCase;
-
 import org.apache.http.HttpResponseInterceptor;
-import org.apache.http.impl.DefaultConnectionReuseStrategy;
-import org.apache.http.impl.DefaultHttpResponseFactory;
-import org.apache.http.impl.nio.DefaultServerIOEventDispatch;
-import org.apache.http.nio.protocol.BufferingHttpServiceHandler;
+import org.apache.http.config.ConnectionConfig;
+import org.apache.http.impl.nio.DefaultHttpServerIODispatch;
+import org.apache.http.nio.protocol.HttpAsyncService;
+import org.apache.http.nio.protocol.UriHttpAsyncRequestHandlerMapper;
 import org.apache.http.nio.reactor.IOEventDispatch;
 import org.apache.http.nio.reactor.IOReactorExceptionHandler;
 import org.apache.http.nio.reactor.IOReactorStatus;
 import org.apache.http.nio.reactor.ListenerEndpoint;
 import org.apache.http.nio.reactor.ListeningIOReactor;
-import org.apache.http.params.HttpParams;
-import org.apache.http.params.SyncBasicHttpParams;
 import org.apache.http.protocol.HttpProcessor;
 import org.apache.http.protocol.ImmutableHttpProcessor;
 import org.apache.http.protocol.ResponseConnControl;
 import org.apache.http.protocol.ResponseContent;
 import org.apache.http.protocol.ResponseDate;
 import org.apache.http.protocol.ResponseServer;
+import org.junit.Assert;
+import org.junit.Test;
 
 /**
  * Basic tests for {@link DefaultListeningIOReactor}.
  */
-public class TestDefaultListeningIOReactor extends TestCase {
+public class TestDefaultListeningIOReactor {
 
-    // ------------------------------------------------------------ Constructor
-    public TestDefaultListeningIOReactor(String testName) {
-        super(testName);
-    }
-
-    // ------------------------------------------------------- TestCase Methods
-
-    public void testEndpointUpAndDown() throws Exception {
-
-        HttpParams params = new SyncBasicHttpParams();
-
-        HttpProcessor httpproc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
+    private static IOEventDispatch createIOEventDispatch() {
+        final HttpProcessor httpproc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
                 new ResponseDate(),
                 new ResponseServer(),
                 new ResponseContent(),
                 new ResponseConnControl()
         });
+        final HttpAsyncService serviceHandler = new HttpAsyncService(httpproc,
+                new UriHttpAsyncRequestHandlerMapper());
+        return new DefaultHttpServerIODispatch(serviceHandler, ConnectionConfig.DEFAULT);
+    }
 
-        final BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
-                httpproc,
-                new DefaultHttpResponseFactory(),
-                new DefaultConnectionReuseStrategy(),
-                params);
+    @Test
+    public void testEndpointUpAndDown() throws Exception {
+        final IOEventDispatch eventDispatch = createIOEventDispatch();
+        final IOReactorConfig config = IOReactorConfig.custom().setIoThreadCount(1).build();
+        final ListeningIOReactor ioreactor = new DefaultListeningIOReactor(config);
 
-        final IOEventDispatch eventDispatch = new DefaultServerIOEventDispatch(
-                serviceHandler,
-                params);
-
-        final ListeningIOReactor ioreactor = new DefaultListeningIOReactor(1, params);
-
-        Thread t = new Thread(new Runnable() {
+        final Thread t = new Thread(new Runnable() {
 
             public void run() {
                 try {
                     ioreactor.execute(eventDispatch);
-                } catch (IOException ex) {
+                } catch (final IOException ex) {
                 }
             }
 
@@ -104,67 +90,51 @@ public class TestDefaultListeningIOReactor extends TestCase {
         t.start();
 
         Set<ListenerEndpoint> endpoints = ioreactor.getEndpoints();
-        assertNotNull(endpoints);
-        assertEquals(0, endpoints.size());
+        Assert.assertNotNull(endpoints);
+        Assert.assertEquals(0, endpoints.size());
 
-        ListenerEndpoint port9998 = ioreactor.listen(new InetSocketAddress(9998));
-        port9998.waitFor();
+        final ListenerEndpoint endpoint1 = ioreactor.listen(new InetSocketAddress(0));
+        endpoint1.waitFor();
 
-        ListenerEndpoint port9999 = ioreactor.listen(new InetSocketAddress(9999));
-        port9999.waitFor();
+        final ListenerEndpoint endpoint2 = ioreactor.listen(new InetSocketAddress(0));
+        endpoint2.waitFor();
+        final int port = ((InetSocketAddress) endpoint2.getAddress()).getPort();
 
         endpoints = ioreactor.getEndpoints();
-        assertNotNull(endpoints);
-        assertEquals(2, endpoints.size());
+        Assert.assertNotNull(endpoints);
+        Assert.assertEquals(2, endpoints.size());
 
-        port9998.close();
+        endpoint1.close();
 
         endpoints = ioreactor.getEndpoints();
-        assertNotNull(endpoints);
-        assertEquals(1, endpoints.size());
+        Assert.assertNotNull(endpoints);
+        Assert.assertEquals(1, endpoints.size());
 
-        ListenerEndpoint endpoint = endpoints.iterator().next();
+        final ListenerEndpoint endpoint = endpoints.iterator().next();
 
-        assertEquals(9999, ((InetSocketAddress) endpoint.getAddress()).getPort());
+        Assert.assertEquals(port, ((InetSocketAddress) endpoint.getAddress()).getPort());
 
         ioreactor.shutdown(1000);
         t.join(1000);
 
-        assertEquals(IOReactorStatus.SHUT_DOWN, ioreactor.getStatus());
+        Assert.assertEquals(IOReactorStatus.SHUT_DOWN, ioreactor.getStatus());
     }
 
+    @Test
     public void testEndpointAlreadyBoundFatal() throws Exception {
-
-        HttpParams params = new SyncBasicHttpParams();
-
-        HttpProcessor httpproc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
-                new ResponseDate(),
-                new ResponseServer(),
-                new ResponseContent(),
-                new ResponseConnControl()
-        });
-
-        final BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
-                httpproc,
-                new DefaultHttpResponseFactory(),
-                new DefaultConnectionReuseStrategy(),
-                params);
-
-        final IOEventDispatch eventDispatch = new DefaultServerIOEventDispatch(
-                serviceHandler,
-                params);
-
-        final ListeningIOReactor ioreactor = new DefaultListeningIOReactor(1, params);
+        final IOEventDispatch eventDispatch = createIOEventDispatch();
+        final IOReactorConfig config = IOReactorConfig.custom().setIoThreadCount(1).build();
+        final ListeningIOReactor ioreactor = new DefaultListeningIOReactor(config);
 
         final CountDownLatch latch = new CountDownLatch(1);
 
-        Thread t = new Thread(new Runnable() {
+        final Thread t = new Thread(new Runnable() {
 
             public void run() {
                 try {
                     ioreactor.execute(eventDispatch);
-                    fail("IOException should have been thrown");
-                } catch (IOException ex) {
+                    Assert.fail("IOException should have been thrown");
+                } catch (final IOException ex) {
                     latch.countDown();
                 }
             }
@@ -173,49 +143,33 @@ public class TestDefaultListeningIOReactor extends TestCase {
 
         t.start();
 
-        ListenerEndpoint endpoint1 = ioreactor.listen(new InetSocketAddress(9999));
+        final ListenerEndpoint endpoint1 = ioreactor.listen(new InetSocketAddress(0));
         endpoint1.waitFor();
+        final int port = ((InetSocketAddress) endpoint1.getAddress()).getPort();
 
-        ListenerEndpoint endpoint2 = ioreactor.listen(new InetSocketAddress(9999));
+        final ListenerEndpoint endpoint2 = ioreactor.listen(new InetSocketAddress(port));
         endpoint2.waitFor();
-        assertNotNull(endpoint2.getException());
+        Assert.assertNotNull(endpoint2.getException());
 
         // I/O reactor is now expected to be shutting down
         latch.await(2000, TimeUnit.MILLISECONDS);
-        assertTrue(ioreactor.getStatus().compareTo(IOReactorStatus.SHUTTING_DOWN) >= 0);
+        Assert.assertTrue(ioreactor.getStatus().compareTo(IOReactorStatus.SHUTTING_DOWN) >= 0);
 
-        Set<ListenerEndpoint> endpoints = ioreactor.getEndpoints();
-        assertNotNull(endpoints);
-        assertEquals(0, endpoints.size());
+        final Set<ListenerEndpoint> endpoints = ioreactor.getEndpoints();
+        Assert.assertNotNull(endpoints);
+        Assert.assertEquals(0, endpoints.size());
 
         ioreactor.shutdown(1000);
         t.join(1000);
 
-        assertEquals(IOReactorStatus.SHUT_DOWN, ioreactor.getStatus());
+        Assert.assertEquals(IOReactorStatus.SHUT_DOWN, ioreactor.getStatus());
     }
 
+    @Test
     public void testEndpointAlreadyBoundNonFatal() throws Exception {
-
-        HttpParams params = new SyncBasicHttpParams();
-
-        HttpProcessor httpproc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
-                new ResponseDate(),
-                new ResponseServer(),
-                new ResponseContent(),
-                new ResponseConnControl()
-        });
-
-        final BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
-                httpproc,
-                new DefaultHttpResponseFactory(),
-                new DefaultConnectionReuseStrategy(),
-                params);
-
-        final IOEventDispatch eventDispatch = new DefaultServerIOEventDispatch(
-                serviceHandler,
-                params);
-
-        final DefaultListeningIOReactor ioreactor = new DefaultListeningIOReactor(1, params);
+        final IOEventDispatch eventDispatch = createIOEventDispatch();
+        final IOReactorConfig config = IOReactorConfig.custom().setIoThreadCount(1).build();
+        final DefaultListeningIOReactor ioreactor = new DefaultListeningIOReactor(config);
 
         ioreactor.setExceptionHandler(new IOReactorExceptionHandler() {
 
@@ -229,12 +183,12 @@ public class TestDefaultListeningIOReactor extends TestCase {
 
         });
 
-        Thread t = new Thread(new Runnable() {
+        final Thread t = new Thread(new Runnable() {
 
             public void run() {
                 try {
                     ioreactor.execute(eventDispatch);
-                } catch (IOException ex) {
+                } catch (final IOException ex) {
                 }
             }
 
@@ -242,22 +196,22 @@ public class TestDefaultListeningIOReactor extends TestCase {
 
         t.start();
 
-        ListenerEndpoint endpoint1 = ioreactor.listen(new InetSocketAddress(9999));
+        final ListenerEndpoint endpoint1 = ioreactor.listen(new InetSocketAddress(9999));
         endpoint1.waitFor();
 
-        ListenerEndpoint endpoint2 = ioreactor.listen(new InetSocketAddress(9999));
+        final ListenerEndpoint endpoint2 = ioreactor.listen(new InetSocketAddress(9999));
         endpoint2.waitFor();
-        assertNotNull(endpoint2.getException());
+        Assert.assertNotNull(endpoint2.getException());
 
         // Sleep a little to make sure the I/O reactor is not shutting down
         Thread.sleep(500);
 
-        assertEquals(IOReactorStatus.ACTIVE, ioreactor.getStatus());
+        Assert.assertEquals(IOReactorStatus.ACTIVE, ioreactor.getStatus());
 
         ioreactor.shutdown(1000);
         t.join(1000);
 
-        assertEquals(IOReactorStatus.SHUT_DOWN, ioreactor.getStatus());
+        Assert.assertEquals(IOReactorStatus.SHUT_DOWN, ioreactor.getStatus());
     }
 
 }
