@@ -170,6 +170,21 @@ public class TestHttpAsyncRequestExecutor {
         Assert.assertEquals(MessageState.COMPLETED, state.getRequestState());
     }
 
+    @Test
+    public void testNullRequest() throws Exception {
+        final State state = new HttpAsyncRequestExecutor.State();
+        this.connContext.setAttribute(HttpAsyncRequestExecutor.HTTP_EXCHANGE_STATE, state);
+        this.connContext.setAttribute(HttpAsyncRequestExecutor.HTTP_HANDLER, this.exchangeHandler);
+        Mockito.when(this.exchangeHandler.generateRequest()).thenReturn(null);
+
+        this.protocolHandler.requestReady(this.conn);
+
+        Mockito.verify(this.exchangeHandler).generateRequest();
+        Assert.assertNull(state.getRequest());
+
+        Mockito.verify(this.conn, Mockito.never()).submitRequest(Mockito.<HttpRequest>any());
+    }
+
     @Test
     public void testEntityEnclosingRequestWithoutExpectContinue() throws Exception {
         final State state = new HttpAsyncRequestExecutor.State();
@@ -479,6 +494,7 @@ public class TestHttpAsyncRequestExecutor {
         this.connContext.setAttribute(HttpAsyncRequestExecutor.HTTP_EXCHANGE_STATE, state);
         this.connContext.setAttribute(HttpAsyncRequestExecutor.HTTP_HANDLER, this.exchangeHandler);
         Mockito.when(this.decoder.isCompleted()).thenReturn(Boolean.TRUE);
+        Mockito.when(this.conn.isOpen()).thenReturn(Boolean.TRUE);
 
         this.protocolHandler.inputReady(this.conn, this.decoder);
 
@@ -489,6 +505,28 @@ public class TestHttpAsyncRequestExecutor {
         Mockito.verify(this.conn).requestOutput();
     }
 
+    @Test
+    public void testResponseContentOutputCompletedHandlerNotDoneConnClosed() throws Exception {
+        final State state = new HttpAsyncRequestExecutor.State();
+        final HttpRequest request = new BasicHttpRequest("GET", "/");
+        state.setRequest(request);
+        final BasicHttpResponse response = new BasicHttpResponse(HttpVersion.HTTP_1_1, 200, "OK");
+        state.setResponse(response);
+        Mockito.when(this.exchangeHandler.isDone()).thenReturn(Boolean.FALSE);
+        this.connContext.setAttribute(HttpAsyncRequestExecutor.HTTP_EXCHANGE_STATE, state);
+        this.connContext.setAttribute(HttpAsyncRequestExecutor.HTTP_HANDLER, this.exchangeHandler);
+        Mockito.when(this.decoder.isCompleted()).thenReturn(Boolean.TRUE);
+        Mockito.when(this.conn.isOpen()).thenReturn(Boolean.FALSE);
+
+        this.protocolHandler.inputReady(this.conn, this.decoder);
+
+        Assert.assertEquals(MessageState.READY, state.getRequestState());
+        Assert.assertEquals(MessageState.READY, state.getResponseState());
+        Mockito.verify(this.exchangeHandler).consumeContent(this.decoder, this.conn);
+        Mockito.verify(this.exchangeHandler).responseCompleted();
+        Mockito.verify(this.conn, Mockito.never()).requestOutput();
+    }
+
     @Test
     public void testResponseInvalidState() throws Exception {
         final State state = new HttpAsyncRequestExecutor.State();
