@@ -31,26 +31,21 @@ import java.io.ByteArrayInputStream;
 import java.net.Socket;
 
 import org.apache.http.ConnectionReuseStrategy;
+import org.apache.http.Consts;
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpHost;
-import org.apache.http.HttpRequestInterceptor;
 import org.apache.http.HttpResponse;
-import org.apache.http.HttpVersion;
 import org.apache.http.entity.ByteArrayEntity;
+import org.apache.http.entity.ContentType;
 import org.apache.http.entity.InputStreamEntity;
 import org.apache.http.entity.StringEntity;
+import org.apache.http.impl.DefaultBHttpClientConnection;
 import org.apache.http.impl.DefaultConnectionReuseStrategy;
-import org.apache.http.impl.DefaultHttpClientConnection;
 import org.apache.http.message.BasicHttpEntityEnclosingRequest;
-import org.apache.http.params.HttpParams;
-import org.apache.http.params.HttpProtocolParams;
-import org.apache.http.params.SyncBasicHttpParams;
-import org.apache.http.protocol.HttpContext;
-import org.apache.http.protocol.BasicHttpContext;
-import org.apache.http.protocol.ExecutionContext;
+import org.apache.http.protocol.HttpCoreContext;
 import org.apache.http.protocol.HttpProcessor;
+import org.apache.http.protocol.HttpProcessorBuilder;
 import org.apache.http.protocol.HttpRequestExecutor;
-import org.apache.http.protocol.ImmutableHttpProcessor;
 import org.apache.http.protocol.RequestConnControl;
 import org.apache.http.protocol.RequestContent;
 import org.apache.http.protocol.RequestExpectContinue;
@@ -59,78 +54,61 @@ import org.apache.http.protocol.RequestUserAgent;
 import org.apache.http.util.EntityUtils;
 
 /**
- * Elemental example for executing a POST request.
- * <p>
- * Please note the purpose of this application is demonstrate the usage of HttpCore APIs.
- * It is NOT intended to demonstrate the most efficient way of building an HTTP client. 
- *
- *
- *
+ * Elemental example for executing multiple POST requests sequentially.
  */
 public class ElementalHttpPost {
 
     public static void main(String[] args) throws Exception {
-        
-        HttpParams params = new SyncBasicHttpParams();
-        HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
-        HttpProtocolParams.setContentCharset(params, "UTF-8");
-        HttpProtocolParams.setUserAgent(params, "HttpComponents/1.1");
-        HttpProtocolParams.setUseExpectContinue(params, true);
-        
-        HttpProcessor httpproc = new ImmutableHttpProcessor(new HttpRequestInterceptor[] {
-                // Required protocol interceptors
-                new RequestContent(),
-                new RequestTargetHost(),
-                // Recommended protocol interceptors
-                new RequestConnControl(),
-                new RequestUserAgent(),
-                new RequestExpectContinue()});
-        
+        HttpProcessor httpproc = HttpProcessorBuilder.create()
+            .add(new RequestContent())
+            .add(new RequestTargetHost())
+            .add(new RequestConnControl())
+            .add(new RequestUserAgent("Test/1.1"))
+            .add(new RequestExpectContinue(true)).build();
+
         HttpRequestExecutor httpexecutor = new HttpRequestExecutor();
 
-        HttpContext context = new BasicHttpContext(null);
-        
+        HttpCoreContext coreContext = HttpCoreContext.create();
         HttpHost host = new HttpHost("localhost", 8080);
-        
-        DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
-        ConnectionReuseStrategy connStrategy = new DefaultConnectionReuseStrategy();
+        coreContext.setTargetHost(host);
 
-        context.setAttribute(ExecutionContext.HTTP_CONNECTION, conn);
-        context.setAttribute(ExecutionContext.HTTP_TARGET_HOST, host);
+        DefaultBHttpClientConnection conn = new DefaultBHttpClientConnection(8 * 1024);
+        ConnectionReuseStrategy connStrategy = DefaultConnectionReuseStrategy.INSTANCE;
 
         try {
-            
+
             HttpEntity[] requestBodies = {
                     new StringEntity(
-                            "This is the first test request", "UTF-8"),
+                            "This is the first test request",
+                            ContentType.create("text/plain", Consts.UTF_8)),
                     new ByteArrayEntity(
-                            "This is the second test request".getBytes("UTF-8")),
+                            "This is the second test request".getBytes("UTF-8"),
+                            ContentType.APPLICATION_OCTET_STREAM),
                     new InputStreamEntity(
                             new ByteArrayInputStream(
                                     "This is the third test request (will be chunked)"
-                                    .getBytes("UTF-8")), -1)
+                                    .getBytes("UTF-8")),
+                            ContentType.APPLICATION_OCTET_STREAM)
             };
-            
+
             for (int i = 0; i < requestBodies.length; i++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
-                    conn.bind(socket, params);
+                    conn.bind(socket);
                 }
-                BasicHttpEntityEnclosingRequest request = new BasicHttpEntityEnclosingRequest("POST", 
+                BasicHttpEntityEnclosingRequest request = new BasicHttpEntityEnclosingRequest("POST",
                         "/servlets-examples/servlet/RequestInfoExample");
                 request.setEntity(requestBodies[i]);
                 System.out.println(">> Request URI: " + request.getRequestLine().getUri());
 
-                request.setParams(params);
-                httpexecutor.preProcess(request, httpproc, context);
-                HttpResponse response = httpexecutor.execute(request, conn, context);
-                response.setParams(params);
-                httpexecutor.postProcess(response, httpproc, context);
-                
+                httpexecutor.preProcess(request, httpproc, coreContext);
+                HttpResponse response = httpexecutor.execute(request, conn, coreContext);
+                httpexecutor.postProcess(response, httpproc, coreContext);
+
                 System.out.println("<< Response: " + response.getStatusLine());
                 System.out.println(EntityUtils.toString(response.getEntity()));
                 System.out.println("==============");
-                if (!connStrategy.keepAlive(response, context)) {
+                if (!connStrategy.keepAlive(response, coreContext)) {
                     conn.close();
                 } else {
                     System.out.println("Connection kept alive...");
@@ -138,7 +116,7 @@ public class ElementalHttpPost {
             }
         } finally {
             conn.close();
-        }        
+        }
     }
-    
+
 }
