@@ -30,53 +30,49 @@ package org.apache.http.examples;
 import java.io.File;
 import java.io.IOException;
 import java.io.InterruptedIOException;
-import java.io.OutputStream;
-import java.io.OutputStreamWriter;
 import java.net.ServerSocket;
 import java.net.Socket;
+import java.net.URL;
 import java.net.URLDecoder;
+import java.nio.charset.Charset;
+import java.security.KeyStore;
 import java.util.Locale;
 
 import org.apache.http.ConnectionClosedException;
+import org.apache.http.HttpConnectionFactory;
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpEntityEnclosingRequest;
 import org.apache.http.HttpException;
 import org.apache.http.HttpRequest;
 import org.apache.http.HttpResponse;
-import org.apache.http.HttpResponseInterceptor;
 import org.apache.http.HttpServerConnection;
 import org.apache.http.HttpStatus;
 import org.apache.http.MethodNotSupportedException;
-import org.apache.http.entity.ContentProducer;
-import org.apache.http.entity.EntityTemplate;
+import org.apache.http.entity.ContentType;
 import org.apache.http.entity.FileEntity;
-import org.apache.http.impl.DefaultConnectionReuseStrategy;
-import org.apache.http.impl.DefaultHttpResponseFactory;
-import org.apache.http.impl.DefaultHttpServerConnection;
-import org.apache.http.params.CoreConnectionPNames;
-import org.apache.http.params.HttpParams;
-import org.apache.http.params.CoreProtocolPNames;
-import org.apache.http.params.SyncBasicHttpParams;
-import org.apache.http.protocol.HttpContext;
+import org.apache.http.entity.StringEntity;
+import org.apache.http.impl.DefaultBHttpServerConnection;
+import org.apache.http.impl.DefaultBHttpServerConnectionFactory;
 import org.apache.http.protocol.BasicHttpContext;
+import org.apache.http.protocol.HttpContext;
 import org.apache.http.protocol.HttpProcessor;
+import org.apache.http.protocol.HttpProcessorBuilder;
 import org.apache.http.protocol.HttpRequestHandler;
-import org.apache.http.protocol.HttpRequestHandlerRegistry;
 import org.apache.http.protocol.HttpService;
-import org.apache.http.protocol.ImmutableHttpProcessor;
 import org.apache.http.protocol.ResponseConnControl;
 import org.apache.http.protocol.ResponseContent;
 import org.apache.http.protocol.ResponseDate;
 import org.apache.http.protocol.ResponseServer;
+import org.apache.http.protocol.UriHttpRequestHandlerMapper;
 import org.apache.http.util.EntityUtils;
 
+import javax.net.ssl.KeyManager;
+import javax.net.ssl.KeyManagerFactory;
+import javax.net.ssl.SSLContext;
+import javax.net.ssl.SSLServerSocketFactory;
+
 /**
  * Basic, yet fully functional and spec compliant, HTTP/1.1 file server.
- * <p>
- * Please note the purpose of this application is demonstrate the usage of HttpCore APIs.
- * It is NOT intended to demonstrate the most efficient way of building an HTTP file server. 
- * 
- *
  */
 public class ElementalHttpServer {
 
@@ -85,28 +81,69 @@ public class ElementalHttpServer {
             System.err.println("Please specify document root directory");
             System.exit(1);
         }
-        Thread t = new RequestListenerThread(8080, args[0]);
+        // Document root directory
+        String docRoot = args[0];
+        int port = 8080;
+        if (args.length >= 2) {
+            port = Integer.parseInt(args[1]);
+        }
+
+        // Set up the HTTP protocol processor
+        HttpProcessor httpproc = HttpProcessorBuilder.create()
+                .add(new ResponseDate())
+                .add(new ResponseServer("Test/1.1"))
+                .add(new ResponseContent())
+                .add(new ResponseConnControl()).build();
+
+        // Set up request handlers
+        UriHttpRequestHandlerMapper reqistry = new UriHttpRequestHandlerMapper();
+        reqistry.register("*", new HttpFileHandler(docRoot));
+
+        // Set up the HTTP service
+        HttpService httpService = new HttpService(httpproc, reqistry);
+
+        SSLServerSocketFactory sf = null;
+        if (port == 8443) {
+            // Initialize SSL context
+            ClassLoader cl = ElementalHttpServer.class.getClassLoader();
+            URL url = cl.getResource("my.keystore");
+            if (url == null) {
+                System.out.println("Keystore not found");
+                System.exit(1);
+            }
+            KeyStore keystore  = KeyStore.getInstance("jks");
+            keystore.load(url.openStream(), "secret".toCharArray());
+            KeyManagerFactory kmfactory = KeyManagerFactory.getInstance(
+                    KeyManagerFactory.getDefaultAlgorithm());
+            kmfactory.init(keystore, "secret".toCharArray());
+            KeyManager[] keymanagers = kmfactory.getKeyManagers();
+            SSLContext sslcontext = SSLContext.getInstance("TLS");
+            sslcontext.init(keymanagers, null, null);
+            sf = sslcontext.getServerSocketFactory();
+        }
+
+        Thread t = new RequestListenerThread(port, httpService, sf);
         t.setDaemon(false);
         t.start();
     }
-    
+
     static class HttpFileHandler implements HttpRequestHandler  {
-        
+
         private final String docRoot;
-        
+
         public HttpFileHandler(final String docRoot) {
             super();
             this.docRoot = docRoot;
         }
-        
+
         public void handle(
-                final HttpRequest request, 
+                final HttpRequest request,
                 final HttpResponse response,
                 final HttpContext context) throws HttpException, IOException {
 
             String method = request.getRequestLine().getMethod().toUpperCase(Locale.ENGLISH);
             if (!method.equals("GET") && !method.equals("HEAD") && !method.equals("POST")) {
-                throw new MethodNotSupportedException(method + " method not supported"); 
+                throw new MethodNotSupportedException(method + " method not supported");
             }
             String target = request.getRequestLine().getUri();
 
@@ -115,104 +152,62 @@ public class ElementalHttpServer {
                 byte[] entityContent = EntityUtils.toByteArray(entity);
                 System.out.println("Incoming entity content (bytes): " + entityContent.length);
             }
-            
-            final File file = new File(this.docRoot, URLDecoder.decode(target));
+
+            final File file = new File(this.docRoot, URLDecoder.decode(target, "UTF-8"));
             if (!file.exists()) {
 
                 response.setStatusCode(HttpStatus.SC_NOT_FOUND);
-                EntityTemplate body = new EntityTemplate(new ContentProducer() {
-                    
-                    public void writeTo(final OutputStream outstream) throws IOException {
-                        OutputStreamWriter writer = new OutputStreamWriter(outstream, "UTF-8"); 
-                        writer.write("<html><body><h1>");
-                        writer.write("File ");
-                        writer.write(file.getPath());
-                        writer.write(" not found");
-                        writer.write("</h1></body></html>");
-                        writer.flush();
-                    }
-                    
-                });
-                body.setContentType("text/html; charset=UTF-8");
-                response.setEntity(body);
+                StringEntity entity = new StringEntity(
+                        "<html><body><h1>File" + file.getPath() +
+                        " not found</h1></body></html>",
+                        ContentType.create("text/html", "UTF-8"));
+                response.setEntity(entity);
                 System.out.println("File " + file.getPath() + " not found");
-                
+
             } else if (!file.canRead() || file.isDirectory()) {
-                
+
                 response.setStatusCode(HttpStatus.SC_FORBIDDEN);
-                EntityTemplate body = new EntityTemplate(new ContentProducer() {
-                    
-                    public void writeTo(final OutputStream outstream) throws IOException {
-                        OutputStreamWriter writer = new OutputStreamWriter(outstream, "UTF-8"); 
-                        writer.write("<html><body><h1>");
-                        writer.write("Access denied");
-                        writer.write("</h1></body></html>");
-                        writer.flush();
-                    }
-                    
-                });
-                body.setContentType("text/html; charset=UTF-8");
-                response.setEntity(body);
+                StringEntity entity = new StringEntity(
+                        "<html><body><h1>Access denied</h1></body></html>",
+                        ContentType.create("text/html", "UTF-8"));
+                response.setEntity(entity);
                 System.out.println("Cannot read file " + file.getPath());
-                
+
             } else {
-                
+
                 response.setStatusCode(HttpStatus.SC_OK);
-                FileEntity body = new FileEntity(file, "text/html");
+                FileEntity body = new FileEntity(file, ContentType.create("text/html", (Charset) null));
                 response.setEntity(body);
                 System.out.println("Serving file " + file.getPath());
-                
             }
         }
-        
+
     }
-    
+
     static class RequestListenerThread extends Thread {
 
+        private final HttpConnectionFactory<DefaultBHttpServerConnection> connFactory;
         private final ServerSocket serversocket;
-        private final HttpParams params; 
         private final HttpService httpService;
-        
-        public RequestListenerThread(int port, final String docroot) throws IOException {
-            this.serversocket = new ServerSocket(port);
-            this.params = new SyncBasicHttpParams();
-            this.params
-                .setIntParameter(CoreConnectionPNames.SO_TIMEOUT, 5000)
-                .setIntParameter(CoreConnectionPNames.SOCKET_BUFFER_SIZE, 8 * 1024)
-                .setBooleanParameter(CoreConnectionPNames.STALE_CONNECTION_CHECK, false)
-                .setBooleanParameter(CoreConnectionPNames.TCP_NODELAY, true)
-                .setParameter(CoreProtocolPNames.ORIGIN_SERVER, "HttpComponents/1.1");
 
-            // Set up the HTTP protocol processor
-            HttpProcessor httpproc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
-                    new ResponseDate(),
-                    new ResponseServer(),
-                    new ResponseContent(),
-                    new ResponseConnControl()
-            });
-            
-            // Set up request handlers
-            HttpRequestHandlerRegistry reqistry = new HttpRequestHandlerRegistry();
-            reqistry.register("*", new HttpFileHandler(docroot));
-            
-            // Set up the HTTP service
-            this.httpService = new HttpService(
-                    httpproc, 
-                    new DefaultConnectionReuseStrategy(), 
-                    new DefaultHttpResponseFactory(),
-                    reqistry,
-                    this.params);
+        public RequestListenerThread(
+                final int port,
+                final HttpService httpService,
+                final SSLServerSocketFactory sf) throws IOException {
+            this.connFactory = DefaultBHttpServerConnectionFactory.INSTANCE;
+            this.serversocket = sf != null ? sf.createServerSocket(port) : new ServerSocket(port);
+            this.httpService = httpService;
         }
-        
+
+        @Override
         public void run() {
             System.out.println("Listening on port " + this.serversocket.getLocalPort());
             while (!Thread.interrupted()) {
                 try {
                     // Set up HTTP connection
                     Socket socket = this.serversocket.accept();
-                    DefaultHttpServerConnection conn = new DefaultHttpServerConnection();
                     System.out.println("Incoming connection from " + socket.getInetAddress());
-                    conn.bind(socket, this.params);
+                    HttpServerConnection conn = this.connFactory.createConnection(socket);
 
                     // Start worker thread
                     Thread t = new WorkerThread(this.httpService, conn);
@@ -221,27 +216,28 @@ public class ElementalHttpServer {
                 } catch (InterruptedIOException ex) {
                     break;
                 } catch (IOException e) {
-                    System.err.println("I/O error initialising connection thread: " 
+                    System.err.println("I/O error initialising connection thread: "
                             + e.getMessage());
                     break;
                 }
             }
         }
     }
-    
+
     static class WorkerThread extends Thread {
 
         private final HttpService httpservice;
         private final HttpServerConnection conn;
-        
+
         public WorkerThread(
-                final HttpService httpservice, 
+                final HttpService httpservice,
                 final HttpServerConnection conn) {
             super();
             this.httpservice = httpservice;
             this.conn = conn;
         }
-        
+
+        @Override
         public void run() {
             System.out.println("New connection thread");
             HttpContext context = new BasicHttpContext(null);
@@ -263,5 +259,5 @@ public class ElementalHttpServer {
         }
 
     }
-    
+
 }
