@@ -42,22 +42,16 @@ import org.apache.http.HttpRequestInterceptor;
 import org.apache.http.HttpResponse;
 import org.apache.http.HttpResponseInterceptor;
 import org.apache.http.HttpServerConnection;
+import org.apache.http.impl.DefaultBHttpClientConnection;
+import org.apache.http.impl.DefaultBHttpServerConnection;
 import org.apache.http.impl.DefaultConnectionReuseStrategy;
-import org.apache.http.impl.DefaultHttpClientConnection;
-import org.apache.http.impl.DefaultHttpResponseFactory;
-import org.apache.http.impl.DefaultHttpServerConnection;
-import org.apache.http.params.CoreConnectionPNames;
-import org.apache.http.params.CoreProtocolPNames;
-import org.apache.http.params.HttpParams;
-import org.apache.http.params.SyncBasicHttpParams;
 import org.apache.http.protocol.BasicHttpContext;
-import org.apache.http.protocol.ExecutionContext;
 import org.apache.http.protocol.HTTP;
 import org.apache.http.protocol.HttpContext;
+import org.apache.http.protocol.HttpCoreContext;
 import org.apache.http.protocol.HttpProcessor;
 import org.apache.http.protocol.HttpRequestExecutor;
 import org.apache.http.protocol.HttpRequestHandler;
-import org.apache.http.protocol.HttpRequestHandlerRegistry;
 import org.apache.http.protocol.HttpService;
 import org.apache.http.protocol.ImmutableHttpProcessor;
 import org.apache.http.protocol.RequestConnControl;
@@ -69,45 +63,41 @@ import org.apache.http.protocol.ResponseConnControl;
 import org.apache.http.protocol.ResponseContent;
 import org.apache.http.protocol.ResponseDate;
 import org.apache.http.protocol.ResponseServer;
+import org.apache.http.protocol.UriHttpRequestHandlerMapper;
 
 /**
- * Rudimentary HTTP/1.1 reverse proxy.
- * <p>
- * Please note the purpose of this application is demonstrate the usage of HttpCore APIs.
- * It is NOT intended to demonstrate the most efficient way of building an HTTP reverse proxy. 
- * 
- *
+ * Elemental HTTP/1.1 reverse proxy.
  */
 public class ElementalReverseProxy {
 
     private static final String HTTP_IN_CONN = "http.proxy.in-conn";
     private static final String HTTP_OUT_CONN = "http.proxy.out-conn";
     private static final String HTTP_CONN_KEEPALIVE = "http.proxy.conn-keepalive";
-    
-    public static void main(String[] args) throws Exception {
+
+    public static void main(final String[] args) throws Exception {
         if (args.length < 1) {
             System.err.println("Please specified target hostname and port");
             System.exit(1);
         }
-        String hostname = args[0];
+        final String hostname = args[0];
         int port = 80;
         if (args.length > 1) {
             port = Integer.parseInt(args[1]);
         }
-        HttpHost target = new HttpHost(hostname, port);
-        
-        Thread t = new RequestListenerThread(8888, target);
+        final HttpHost target = new HttpHost(hostname, port);
+
+        final Thread t = new RequestListenerThread(8888, target);
         t.setDaemon(false);
         t.start();
     }
-    
+
     static class ProxyHandler implements HttpRequestHandler  {
-        
+
         private final HttpHost target;
         private final HttpProcessor httpproc;
         private final HttpRequestExecutor httpexecutor;
         private final ConnectionReuseStrategy connStrategy;
-        
+
         public ProxyHandler(
                 final HttpHost target,
                 final HttpProcessor httpproc,
@@ -116,19 +106,19 @@ public class ElementalReverseProxy {
             this.target = target;
             this.httpproc = httpproc;
             this.httpexecutor = httpexecutor;
-            this.connStrategy = new DefaultConnectionReuseStrategy();
+            this.connStrategy = DefaultConnectionReuseStrategy.INSTANCE;
         }
-        
+
         public void handle(
-                final HttpRequest request, 
+                final HttpRequest request,
                 final HttpResponse response,
                 final HttpContext context) throws HttpException, IOException {
-            
-            HttpClientConnection conn = (HttpClientConnection) context.getAttribute(
+
+            final HttpClientConnection conn = (HttpClientConnection) context.getAttribute(
                     HTTP_OUT_CONN);
 
-            context.setAttribute(ExecutionContext.HTTP_CONNECTION, conn);
-            context.setAttribute(ExecutionContext.HTTP_TARGET_HOST, this.target);
+            context.setAttribute(HttpCoreContext.HTTP_CONNECTION, conn);
+            context.setAttribute(HttpCoreContext.HTTP_TARGET_HOST, this.target);
 
             System.out.println(">> Request URI: " + request.getRequestLine().getUri());
 
@@ -141,11 +131,11 @@ public class ElementalReverseProxy {
             request.removeHeaders("TE");
             request.removeHeaders("Trailers");
             request.removeHeaders("Upgrade");
-            
+
             this.httpexecutor.preProcess(request, this.httpproc, context);
-            HttpResponse targetResponse = this.httpexecutor.execute(request, conn, context);
+            final HttpResponse targetResponse = this.httpexecutor.execute(request, conn, context);
             this.httpexecutor.postProcess(response, this.httpproc, context);
-            
+
             // Remove hop-by-hop headers
             targetResponse.removeHeaders(HTTP.CONTENT_LEN);
             targetResponse.removeHeaders(HTTP.TRANSFER_ENCODING);
@@ -154,114 +144,103 @@ public class ElementalReverseProxy {
             targetResponse.removeHeaders("TE");
             targetResponse.removeHeaders("Trailers");
             targetResponse.removeHeaders("Upgrade");
-            
+
             response.setStatusLine(targetResponse.getStatusLine());
             response.setHeaders(targetResponse.getAllHeaders());
             response.setEntity(targetResponse.getEntity());
-            
+
             System.out.println("<< Response: " + response.getStatusLine());
 
-            boolean keepalive = this.connStrategy.keepAlive(response, context);
+            final boolean keepalive = this.connStrategy.keepAlive(response, context);
             context.setAttribute(HTTP_CONN_KEEPALIVE, new Boolean(keepalive));
         }
-        
+
     }
-    
+
     static class RequestListenerThread extends Thread {
 
         private final HttpHost target;
         private final ServerSocket serversocket;
-        private final HttpParams params; 
         private final HttpService httpService;
-        
-        public RequestListenerThread(int port, final HttpHost target) throws IOException {
+
+        public RequestListenerThread(final int port, final HttpHost target) throws IOException {
             this.target = target;
             this.serversocket = new ServerSocket(port);
-            this.params = new SyncBasicHttpParams();
-            this.params
-                .setIntParameter(CoreConnectionPNames.SO_TIMEOUT, 5000)
-                .setIntParameter(CoreConnectionPNames.SOCKET_BUFFER_SIZE, 8 * 1024)
-                .setBooleanParameter(CoreConnectionPNames.STALE_CONNECTION_CHECK, false)
-                .setBooleanParameter(CoreConnectionPNames.TCP_NODELAY, true)
-                .setParameter(CoreProtocolPNames.ORIGIN_SERVER, "HttpComponents/1.1");
 
             // Set up HTTP protocol processor for incoming connections
-            HttpProcessor inhttpproc = new ImmutableHttpProcessor(
+            final HttpProcessor inhttpproc = new ImmutableHttpProcessor(
                     new HttpRequestInterceptor[] {
                             new RequestContent(),
                             new RequestTargetHost(),
                             new RequestConnControl(),
-                            new RequestUserAgent(),
-                            new RequestExpectContinue()
+                            new RequestUserAgent("Test/1.1"),
+                            new RequestExpectContinue(true)
              });
-            
+
             // Set up HTTP protocol processor for outgoing connections
-            HttpProcessor outhttpproc = new ImmutableHttpProcessor(
+            final HttpProcessor outhttpproc = new ImmutableHttpProcessor(
                     new HttpResponseInterceptor[] {
                             new ResponseDate(),
-                            new ResponseServer(),
+                            new ResponseServer("Test/1.1"),
                             new ResponseContent(),
                             new ResponseConnControl()
             });
 
-            // Set up outgoing request executor 
-            HttpRequestExecutor httpexecutor = new HttpRequestExecutor();
-            
+            // Set up outgoing request executor
+            final HttpRequestExecutor httpexecutor = new HttpRequestExecutor();
+
             // Set up incoming request handler
-            HttpRequestHandlerRegistry reqistry = new HttpRequestHandlerRegistry();
+            final UriHttpRequestHandlerMapper reqistry = new UriHttpRequestHandlerMapper();
             reqistry.register("*", new ProxyHandler(
-                    this.target, 
-                    outhttpproc, 
+                    this.target,
+                    outhttpproc,
                     httpexecutor));
-            
+
             // Set up the HTTP service
-            this.httpService = new HttpService(
-                    inhttpproc, 
-                    new DefaultConnectionReuseStrategy(), 
-                    new DefaultHttpResponseFactory(),
-                    reqistry,
-                    this.params);
+            this.httpService = new HttpService(inhttpproc, reqistry);
         }
-        
+
+        @Override
         public void run() {
             System.out.println("Listening on port " + this.serversocket.getLocalPort());
             while (!Thread.interrupted()) {
                 try {
+                    final int bufsize = 8 * 1024;
                     // Set up incoming HTTP connection
-                    Socket insocket = this.serversocket.accept();
-                    DefaultHttpServerConnection inconn = new DefaultHttpServerConnection();
+                    final Socket insocket = this.serversocket.accept();
+                    final DefaultBHttpServerConnection inconn = new DefaultBHttpServerConnection(bufsize);
                     System.out.println("Incoming connection from " + insocket.getInetAddress());
-                    inconn.bind(insocket, this.params);
+                    inconn.bind(insocket);
 
                     // Set up outgoing HTTP connection
-                    Socket outsocket = new Socket(this.target.getHostName(), this.target.getPort());
-                    DefaultHttpClientConnection outconn = new DefaultHttpClientConnection();
-                    outconn.bind(outsocket, this.params);
+                    final Socket outsocket = new Socket(this.target.getHostName(), this.target.getPort());
+                    final DefaultBHttpClientConnection outconn = new DefaultBHttpClientConnection(bufsize);
+                    outconn.bind(outsocket);
                     System.out.println("Outgoing connection to " + outsocket.getInetAddress());
-                    
+
                     // Start worker thread
-                    Thread t = new ProxyThread(this.httpService, inconn, outconn);
+                    final Thread t = new ProxyThread(this.httpService, inconn, outconn);
                     t.setDaemon(true);
                     t.start();
-                } catch (InterruptedIOException ex) {
+                } catch (final InterruptedIOException ex) {
                     break;
-                } catch (IOException e) {
-                    System.err.println("I/O error initialising connection thread: " 
+                } catch (final IOException e) {
+                    System.err.println("I/O error initialising connection thread: "
                             + e.getMessage());
                     break;
                 }
             }
         }
     }
-    
+
     static class ProxyThread extends Thread {
 
         private final HttpService httpservice;
         private final HttpServerConnection inconn;
         private final HttpClientConnection outconn;
-        
+
         public ProxyThread(
-                final HttpService httpservice, 
+                final HttpService httpservice,
                 final HttpServerConnection inconn,
                 final HttpClientConnection outconn) {
             super();
@@ -269,47 +248,48 @@ public class ElementalReverseProxy {
             this.inconn = inconn;
             this.outconn = outconn;
         }
-        
+
+        @Override
         public void run() {
             System.out.println("New connection thread");
-            HttpContext context = new BasicHttpContext(null);
-            
+            final HttpContext context = new BasicHttpContext(null);
+
             // Bind connection objects to the execution context
             context.setAttribute(HTTP_IN_CONN, this.inconn);
             context.setAttribute(HTTP_OUT_CONN, this.outconn);
-            
+
             try {
                 while (!Thread.interrupted()) {
                     if (!this.inconn.isOpen()) {
                         this.outconn.close();
                         break;
                     }
-                    
+
                     this.httpservice.handleRequest(this.inconn, context);
-                    
-                    Boolean keepalive = (Boolean) context.getAttribute(HTTP_CONN_KEEPALIVE);
+
+                    final Boolean keepalive = (Boolean) context.getAttribute(HTTP_CONN_KEEPALIVE);
                     if (!Boolean.TRUE.equals(keepalive)) {
                         this.outconn.close();
                         this.inconn.close();
                         break;
                     }
                 }
-            } catch (ConnectionClosedException ex) {
+            } catch (final ConnectionClosedException ex) {
                 System.err.println("Client closed connection");
-            } catch (IOException ex) {
+            } catch (final IOException ex) {
                 System.err.println("I/O error: " + ex.getMessage());
-            } catch (HttpException ex) {
+            } catch (final HttpException ex) {
                 System.err.println("Unrecoverable HTTP protocol violation: " + ex.getMessage());
             } finally {
                 try {
                     this.inconn.shutdown();
-                } catch (IOException ignore) {}
+                } catch (final IOException ignore) {}
                 try {
                     this.outconn.shutdown();
-                } catch (IOException ignore) {}
+                } catch (final IOException ignore) {}
             }
         }
 
     }
-    
+
 }
