@@ -31,13 +31,16 @@ import org.apache.http.ConnectionReuseStrategy;
 import org.apache.http.Header;
 import org.apache.http.HeaderIterator;
 import org.apache.http.HttpResponse;
+import org.apache.http.HttpStatus;
 import org.apache.http.HttpVersion;
 import org.apache.http.ParseException;
 import org.apache.http.ProtocolVersion;
+import org.apache.http.TokenIterator;
+import org.apache.http.annotation.Immutable;
+import org.apache.http.message.BasicTokenIterator;
 import org.apache.http.protocol.HTTP;
 import org.apache.http.protocol.HttpContext;
-import org.apache.http.TokenIterator;
-import org.apache.http.message.BasicTokenIterator;
+import org.apache.http.util.Args;
 
 /**
  * Default implementation of a strategy deciding about connection re-use.
@@ -57,8 +60,11 @@ import org.apache.http.message.BasicTokenIterator;
  *
  * @since 4.0
  */
+@Immutable
 public class DefaultConnectionReuseStrategy implements ConnectionReuseStrategy {
 
+    public static final DefaultConnectionReuseStrategy INSTANCE = new DefaultConnectionReuseStrategy();
+
     public DefaultConnectionReuseStrategy() {
         super();
     }
@@ -66,37 +72,34 @@ public class DefaultConnectionReuseStrategy implements ConnectionReuseStrategy {
     // see interface ConnectionReuseStrategy
     public boolean keepAlive(final HttpResponse response,
                              final HttpContext context) {
-        if (response == null) {
-            throw new IllegalArgumentException
-                ("HTTP response may not be null.");
-        }
-        if (context == null) {
-            throw new IllegalArgumentException
-                ("HTTP context may not be null.");
-        }
+        Args.notNull(response, "HTTP response");
+        Args.notNull(context, "HTTP context");
 
         // Check for a self-terminating entity. If the end of the entity will
         // be indicated by closing the connection, there is no keep-alive.
-        ProtocolVersion ver = response.getStatusLine().getProtocolVersion();
-        Header teh = response.getFirstHeader(HTTP.TRANSFER_ENCODING);
+        final ProtocolVersion ver = response.getStatusLine().getProtocolVersion();
+        final Header teh = response.getFirstHeader(HTTP.TRANSFER_ENCODING);
         if (teh != null) {
             if (!HTTP.CHUNK_CODING.equalsIgnoreCase(teh.getValue())) {
                 return false;
             }
         } else {
-            Header[] clhs = response.getHeaders(HTTP.CONTENT_LEN);
-            // Do not reuse if not properly content-length delimited
-            if (clhs == null || clhs.length != 1) {
-                return false;
-            }
-            Header clh = clhs[0];
-            try {
-                int contentLen = Integer.parseInt(clh.getValue());
-                if (contentLen < 0) {
+            if (canResponseHaveBody(response)) {
+                final Header[] clhs = response.getHeaders(HTTP.CONTENT_LEN);
+                // Do not reuse if not properly content-length delimited
+                if (clhs.length == 1) {
+                    final Header clh = clhs[0];
+                    try {
+                        final int contentLen = Integer.parseInt(clh.getValue());
+                        if (contentLen < 0) {
+                            return false;
+                        }
+                    } catch (final NumberFormatException ex) {
+                        return false;
+                    }
+                } else {
                     return false;
                 }
-            } catch (NumberFormatException ex) {
-                return false;
             }
         }
 
@@ -104,8 +107,9 @@ public class DefaultConnectionReuseStrategy implements ConnectionReuseStrategy {
         // the "Proxy-Connection" header. The latter is an unspecified and
         // broken but unfortunately common extension of HTTP.
         HeaderIterator hit = response.headerIterator(HTTP.CONN_DIRECTIVE);
-        if (!hit.hasNext())
+        if (!hit.hasNext()) {
             hit = response.headerIterator("Proxy-Connection");
+        }
 
         // Experimental usage of the "Connection" header in HTTP/1.0 is
         // documented in RFC 2068, section 19.7.1. A token "keep-alive" is
@@ -132,7 +136,7 @@ public class DefaultConnectionReuseStrategy implements ConnectionReuseStrategy {
 
         if (hit.hasNext()) {
             try {
-                TokenIterator ti = createTokenIterator(hit);
+                final TokenIterator ti = createTokenIterator(hit);
                 boolean keepalive = false;
                 while (ti.hasNext()) {
                     final String token = ti.nextToken();
@@ -144,10 +148,12 @@ public class DefaultConnectionReuseStrategy implements ConnectionReuseStrategy {
                     }
                 }
                 if (keepalive)
+                 {
                     return true;
                 // neither "close" nor "keep-alive", use default policy
+                }
 
-            } catch (ParseException px) {
+            } catch (final ParseException px) {
                 // invalid connection header means no persistent connection
                 // we don't have logging in HttpCore, so the exception is lost
                 return false;
@@ -168,7 +174,16 @@ public class DefaultConnectionReuseStrategy implements ConnectionReuseStrategy {
      *
      * @return  the token iterator
      */
-    protected TokenIterator createTokenIterator(HeaderIterator hit) {
+    protected TokenIterator createTokenIterator(final HeaderIterator hit) {
         return new BasicTokenIterator(hit);
     }
+
+    private boolean canResponseHaveBody(final HttpResponse response) {
+        final int status = response.getStatusLine().getStatusCode();
+        return status >= HttpStatus.SC_OK
+            && status != HttpStatus.SC_NO_CONTENT
+            && status != HttpStatus.SC_NOT_MODIFIED
+            && status != HttpStatus.SC_RESET_CONTENT;
+    }
+
 }
