@@ -31,82 +31,81 @@ import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.InputStream;
 
-import org.apache.http.protocol.HTTP;
-
-import junit.framework.TestCase;
+import org.apache.http.Consts;
+import org.junit.Assert;
+import org.junit.Test;
 
 /**
  * Unit tests for {@link BasicHttpEntity}.
  *
  */
-public class TestBasicHttpEntity extends TestCase {
-
-    public TestBasicHttpEntity(String testName) {
-        super(testName);
-    }
+public class TestBasicHttpEntity {
 
+    @Test
     public void testBasics() throws Exception {
 
-        byte[] bytes = "Message content".getBytes(HTTP.US_ASCII);
-        InputStream content = new ByteArrayInputStream(bytes);
-        BasicHttpEntity httpentity = new BasicHttpEntity();
+        final byte[] bytes = "Message content".getBytes(Consts.ASCII.name());
+        final InputStream content = new ByteArrayInputStream(bytes);
+        final BasicHttpEntity httpentity = new BasicHttpEntity();
         httpentity.setContent(content);
         httpentity.setContentLength(bytes.length);
 
-        assertEquals(bytes.length, httpentity.getContentLength());
-        assertFalse(httpentity.isRepeatable());
-        assertTrue(httpentity.isStreaming());
+        Assert.assertEquals(bytes.length, httpentity.getContentLength());
+        Assert.assertFalse(httpentity.isRepeatable());
+        Assert.assertTrue(httpentity.isStreaming());
     }
 
+    @Test
     public void testContent() throws Exception {
-        byte[] bytes = "Message content".getBytes(HTTP.US_ASCII);
-        InputStream content = new ByteArrayInputStream(bytes);
-        BasicHttpEntity httpentity = new BasicHttpEntity();
+        final byte[] bytes = "Message content".getBytes(Consts.ASCII.name());
+        final InputStream content = new ByteArrayInputStream(bytes);
+        final BasicHttpEntity httpentity = new BasicHttpEntity();
         try {
             httpentity.getContent();
-            fail("IllegalStateException should have been thrown");
-        } catch (IllegalStateException ex) {
+            Assert.fail("IllegalStateException should have been thrown");
+        } catch (final IllegalStateException ex) {
             // expected
         }
         httpentity.setContent(content);
-        assertEquals(content, httpentity.getContent());
+        Assert.assertEquals(content, httpentity.getContent());
 
         httpentity.setContent(null);
         try {
             httpentity.getContent();
-            fail("IllegalStateException should have been thrown");
-        } catch (IllegalStateException ex) {
+            Assert.fail("IllegalStateException should have been thrown");
+        } catch (final IllegalStateException ex) {
             // expected
         }
     }
 
+    @Test
     public void testWriteTo() throws Exception {
-        byte[] bytes = "Message content".getBytes(HTTP.US_ASCII);
-        InputStream content = new ByteArrayInputStream(bytes);
-        BasicHttpEntity httpentity = new BasicHttpEntity();
+        final byte[] bytes = "Message content".getBytes(Consts.ASCII.name());
+        final InputStream content = new ByteArrayInputStream(bytes);
+        final BasicHttpEntity httpentity = new BasicHttpEntity();
         httpentity.setContent(content);
 
         ByteArrayOutputStream out = new ByteArrayOutputStream();
         httpentity.writeTo(out);
-        byte[] bytes2 = out.toByteArray();
-        assertNotNull(bytes2);
-        assertEquals(bytes.length, bytes2.length);
+        final byte[] bytes2 = out.toByteArray();
+        Assert.assertNotNull(bytes2);
+        Assert.assertEquals(bytes.length, bytes2.length);
         for (int i = 0; i < bytes.length; i++) {
-            assertEquals(bytes[i], bytes2[i]);
+            Assert.assertEquals(bytes[i], bytes2[i]);
         }
         httpentity.setContent(null);
         out = new ByteArrayOutputStream();
         try {
             httpentity.writeTo(out);
-            fail("IllegalStateException should have been thrown");
-        } catch (IllegalStateException ex) {
+            Assert.fail("IllegalStateException should have been thrown");
+        } catch (final IllegalStateException ex) {
             // expected
         }
 
         try {
             httpentity.writeTo(null);
-            fail("IllegalArgumentException should have been thrown");
-        } catch (IllegalArgumentException ex) {
+            Assert.fail("IllegalArgumentException should have been thrown");
+        } catch (final IllegalArgumentException ex) {
             // expected
         }
     }
