@@ -35,6 +35,7 @@ import java.io.OutputStream;
 
 import org.apache.http.Header;
 import org.apache.http.MalformedChunkCodingException;
+import org.apache.http.TruncatedChunkException;
 import org.apache.http.impl.SessionInputBufferMock;
 import org.apache.http.impl.SessionOutputBufferMock;
 import org.apache.http.io.SessionInputBuffer;
@@ -51,7 +52,7 @@ public class TestChunkCoding {
         try {
             new ChunkedInputStream((SessionInputBuffer)null);
             Assert.fail("IllegalArgumentException should have been thrown");
-        } catch (IllegalArgumentException ex) {
+        } catch (final IllegalArgumentException ex) {
             // expected
         }
         new MalformedChunkCodingException();
@@ -67,11 +68,11 @@ public class TestChunkCoding {
     // Test for when buffer is larger than chunk size
     @Test
     public void testChunkedInputStreamLargeBuffer() throws IOException {
-        ChunkedInputStream in = new ChunkedInputStream(
+        final ChunkedInputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(CHUNKED_INPUT, CONTENT_CHARSET)));
-        byte[] buffer = new byte[300];
-        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        final byte[] buffer = new byte[300];
+        final ByteArrayOutputStream out = new ByteArrayOutputStream();
         int len;
         while ((len = in.read(buffer)) > 0) {
             out.write(buffer, 0, len);
@@ -81,10 +82,10 @@ public class TestChunkCoding {
 
         in.close();
 
-        String result = EncodingUtils.getString(out.toByteArray(), CONTENT_CHARSET);
+        final String result = EncodingUtils.getString(out.toByteArray(), CONTENT_CHARSET);
         Assert.assertEquals(result, CHUNKED_RESULT);
 
-        Header[] footers = in.getFooters();
+        final Header[] footers = in.getFooters();
         Assert.assertNotNull(footers);
         Assert.assertEquals(2, footers.length);
         Assert.assertEquals("Footer1", footers[0].getName());
@@ -96,12 +97,12 @@ public class TestChunkCoding {
     //Test for when buffer is smaller than chunk size.
     @Test
     public void testChunkedInputStreamSmallBuffer() throws IOException {
-        ChunkedInputStream in = new ChunkedInputStream(
+        final ChunkedInputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                             EncodingUtils.getBytes(CHUNKED_INPUT, CONTENT_CHARSET)));
 
-        byte[] buffer = new byte[7];
-        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        final byte[] buffer = new byte[7];
+        final ByteArrayOutputStream out = new ByteArrayOutputStream();
         int len;
         while ((len = in.read(buffer)) > 0) {
             out.write(buffer, 0, len);
@@ -112,7 +113,7 @@ public class TestChunkCoding {
         in.close();
 
         EncodingUtils.getString(out.toByteArray(), CONTENT_CHARSET);
-        Header[] footers = in.getFooters();
+        final Header[] footers = in.getFooters();
         Assert.assertNotNull(footers);
         Assert.assertEquals(2, footers.length);
         Assert.assertEquals("Footer1", footers[0].getName());
@@ -124,8 +125,8 @@ public class TestChunkCoding {
     // One byte read
     @Test
     public void testChunkedInputStreamOneByteRead() throws IOException {
-        String s = "5\r\n01234\r\n5\r\n56789\r\n0\r\n";
-        ChunkedInputStream in = new ChunkedInputStream(
+        final String s = "5\r\n01234\r\n5\r\n56789\r\n0\r\n";
+        final ChunkedInputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         int ch;
@@ -142,19 +143,20 @@ public class TestChunkCoding {
 
     @Test
     public void testAvailable() throws IOException {
-        String s = "5\r\n12345\r\n0\r\n";
-        ChunkedInputStream in = new ChunkedInputStream(
+        final String s = "5\r\n12345\r\n0\r\n";
+        final ChunkedInputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         Assert.assertEquals(0, in.available());
         in.read();
         Assert.assertEquals(4, in.available());
+        in.close();
     }
 
     @Test
     public void testChunkedInputStreamClose() throws IOException {
-        String s = "5\r\n01234\r\n5\r\n56789\r\n0\r\n";
-        ChunkedInputStream in = new ChunkedInputStream(
+        final String s = "5\r\n01234\r\n5\r\n56789\r\n0\r\n";
+        final ChunkedInputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         in.close();
@@ -162,40 +164,40 @@ public class TestChunkCoding {
         try {
             in.read();
             Assert.fail("IOException should have been thrown");
-        } catch (IOException ex) {
+        } catch (final IOException ex) {
             // expected
         }
-        byte[] tmp = new byte[10];
+        final byte[] tmp = new byte[10];
         try {
             in.read(tmp);
             Assert.fail("IOException should have been thrown");
-        } catch (IOException ex) {
+        } catch (final IOException ex) {
             // expected
         }
         try {
             in.read(tmp, 0, tmp.length);
             Assert.fail("IOException should have been thrown");
-        } catch (IOException ex) {
+        } catch (final IOException ex) {
             // expected
         }
     }
 
     @Test
     public void testChunkedOutputStreamClose() throws IOException {
-        ChunkedOutputStream out = new ChunkedOutputStream(
-                new SessionOutputBufferMock());
+        final ChunkedOutputStream out = new ChunkedOutputStream(
+                2048, new SessionOutputBufferMock());
         out.close();
         out.close();
         try {
             out.write(new byte[] {1,2,3});
             Assert.fail("IOException should have been thrown");
-        } catch (IOException ex) {
+        } catch (final IOException ex) {
             // expected
         }
         try {
             out.write(1);
             Assert.fail("IOException should have been thrown");
-        } catch (IOException ex) {
+        } catch (final IOException ex) {
             // expected
         }
     }
@@ -203,31 +205,32 @@ public class TestChunkCoding {
     // Missing closing chunk
     @Test
     public void testChunkedInputStreamNoClosingChunk() throws IOException {
-        String s = "5\r\n01234\r\n";
-        ChunkedInputStream in = new ChunkedInputStream(
+        final String s = "5\r\n01234\r\n";
+        final ChunkedInputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
-        byte[] tmp = new byte[5];
+        final byte[] tmp = new byte[5];
         Assert.assertEquals(5, in.read(tmp));
         Assert.assertEquals(-1, in.read());
-    }
+        in.close();
+}
 
     // Missing \r\n at the end of the first chunk
     @Test
     public void testCorruptChunkedInputStreamMissingCRLF() throws IOException {
-        String s = "5\r\n012345\r\n56789\r\n0\r\n";
-        InputStream in = new ChunkedInputStream(
+        final String s = "5\r\n012345\r\n56789\r\n0\r\n";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
-        byte[] buffer = new byte[300];
-        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        final byte[] buffer = new byte[300];
+        final ByteArrayOutputStream out = new ByteArrayOutputStream();
         int len;
         try {
             while ((len = in.read(buffer)) > 0) {
                 out.write(buffer, 0, len);
             }
             Assert.fail("MalformedChunkCodingException should have been thrown");
-        } catch(MalformedChunkCodingException e) {
+        } catch(final MalformedChunkCodingException e) {
             /* expected exception */
         }
     }
@@ -235,125 +238,138 @@ public class TestChunkCoding {
     // Missing LF
     @Test
     public void testCorruptChunkedInputStreamMissingLF() throws IOException {
-        String s = "5\r01234\r\n5\r\n56789\r\n0\r\n";
-        InputStream in = new ChunkedInputStream(
+        final String s = "5\r01234\r\n5\r\n56789\r\n0\r\n";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         try {
             in.read();
             Assert.fail("MalformedChunkCodingException should have been thrown");
-        } catch(MalformedChunkCodingException e) {
+        } catch(final MalformedChunkCodingException e) {
             /* expected exception */
         }
-    }
+        in.close();
+}
 
     // Invalid chunk size
     @Test
     public void testCorruptChunkedInputStreamInvalidSize() throws IOException {
-        String s = "whatever\r\n01234\r\n5\r\n56789\r\n0\r\n";
-        InputStream in = new ChunkedInputStream(
+        final String s = "whatever\r\n01234\r\n5\r\n56789\r\n0\r\n";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         try {
             in.read();
             Assert.fail("MalformedChunkCodingException should have been thrown");
-        } catch(MalformedChunkCodingException e) {
+        } catch(final MalformedChunkCodingException e) {
             /* expected exception */
         }
-    }
+        try {
+            in.close();
+        } catch (TruncatedChunkException expected) {
+        }
+}
 
     // Negative chunk size
     @Test
     public void testCorruptChunkedInputStreamNegativeSize() throws IOException {
-        String s = "-5\r\n01234\r\n5\r\n56789\r\n0\r\n";
-        InputStream in = new ChunkedInputStream(
+        final String s = "-5\r\n01234\r\n5\r\n56789\r\n0\r\n";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         try {
             in.read();
             Assert.fail("MalformedChunkCodingException should have been thrown");
-        } catch(MalformedChunkCodingException e) {
+        } catch(final MalformedChunkCodingException e) {
             /* expected exception */
         }
-    }
+        try {
+            in.close();
+        } catch (TruncatedChunkException expected) {
+        }
+}
 
     // Truncated chunk
     @Test
     public void testCorruptChunkedInputStreamTruncatedChunk() throws IOException {
-        String s = "3\r\n12";
-        InputStream in = new ChunkedInputStream(
+        final String s = "3\r\n12";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
-        byte[] buffer = new byte[300];
+        final byte[] buffer = new byte[300];
         Assert.assertEquals(2, in.read(buffer));
         try {
             in.read(buffer);
             Assert.fail("MalformedChunkCodingException should have been thrown");
-        } catch(MalformedChunkCodingException e) {
+        } catch(final MalformedChunkCodingException e) {
             /* expected exception */
         }
-    }
+        in.close();
+}
 
     // Invalid footer
     @Test
     public void testCorruptChunkedInputStreamInvalidFooter() throws IOException {
-        String s = "1\r\n0\r\n0\r\nstuff\r\n";
-        InputStream in = new ChunkedInputStream(
+        final String s = "1\r\n0\r\n0\r\nstuff\r\n";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(s, CONTENT_CHARSET)));
         try {
             in.read();
             in.read();
             Assert.fail("MalformedChunkCodingException should have been thrown");
-        } catch(MalformedChunkCodingException e) {
+        } catch(final MalformedChunkCodingException e) {
             /* expected exception */
         }
-    }
+        in.close();
+}
 
     @Test
     public void testEmptyChunkedInputStream() throws IOException {
-        String input = "0\r\n";
-        InputStream in = new ChunkedInputStream(
+        final String input = "0\r\n";
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         EncodingUtils.getBytes(input, CONTENT_CHARSET)));
-        byte[] buffer = new byte[300];
-        ByteArrayOutputStream out = new ByteArrayOutputStream();
+        final byte[] buffer = new byte[300];
+        final ByteArrayOutputStream out = new ByteArrayOutputStream();
         int len;
         while ((len = in.read(buffer)) > 0) {
             out.write(buffer, 0, len);
         }
         Assert.assertEquals(0, out.size());
-    }
+        in.close();
+}
 
     @Test
     public void testChunkedConsistence() throws IOException {
-        String input = "76126;27823abcd;:q38a-\nkjc\rk%1ad\tkh/asdui\r\njkh+?\\suweb";
-        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        OutputStream out = new ChunkedOutputStream(new SessionOutputBufferMock(buffer));
+        final String input = "76126;27823abcd;:q38a-\nkjc\rk%1ad\tkh/asdui\r\njkh+?\\suweb";
+        final ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        final OutputStream out = new ChunkedOutputStream(2048, new SessionOutputBufferMock(buffer));
         out.write(EncodingUtils.getBytes(input, CONTENT_CHARSET));
         out.flush();
         out.close();
         out.close();
         buffer.close();
-        InputStream in = new ChunkedInputStream(
+        final InputStream in = new ChunkedInputStream(
                 new SessionInputBufferMock(
                         buffer.toByteArray()));
 
-        byte[] d = new byte[10];
-        ByteArrayOutputStream result = new ByteArrayOutputStream();
+        final byte[] d = new byte[10];
+        final ByteArrayOutputStream result = new ByteArrayOutputStream();
         int len = 0;
         while ((len = in.read(d)) > 0) {
             result.write(d, 0, len);
         }
 
-        String output = EncodingUtils.getString(result.toByteArray(), CONTENT_CHARSET);
+        final String output = EncodingUtils.getString(result.toByteArray(), CONTENT_CHARSET);
         Assert.assertEquals(input, output);
-    }
+        in.close();
+}
 
     @Test
     public void testChunkedOutputStream() throws IOException {
-        SessionOutputBufferMock buffer = new SessionOutputBufferMock();
-        ChunkedOutputStream out = new ChunkedOutputStream(buffer, 2);
+        final SessionOutputBufferMock buffer = new SessionOutputBufferMock();
+        final ChunkedOutputStream out = new ChunkedOutputStream(2, buffer);
         out.write('1');
         out.write('2');
         out.write('3');
@@ -361,7 +377,7 @@ public class TestChunkCoding {
         out.finish();
         out.close();
 
-        byte [] rawdata =  buffer.getData();
+        final byte [] rawdata =  buffer.getData();
 
         Assert.assertEquals(19, rawdata.length);
         Assert.assertEquals('2', rawdata[0]);
@@ -387,13 +403,13 @@ public class TestChunkCoding {
 
     @Test
     public void testChunkedOutputStreamLargeChunk() throws IOException {
-        SessionOutputBufferMock buffer = new SessionOutputBufferMock();
-        ChunkedOutputStream out = new ChunkedOutputStream(buffer, 2);
+        final SessionOutputBufferMock buffer = new SessionOutputBufferMock();
+        final ChunkedOutputStream out = new ChunkedOutputStream(2, buffer);
         out.write(new byte[] {'1', '2', '3', '4'});
         out.finish();
         out.close();
 
-        byte [] rawdata =  buffer.getData();
+        final byte [] rawdata =  buffer.getData();
 
         Assert.assertEquals(14, rawdata.length);
         Assert.assertEquals('4', rawdata[0]);
@@ -414,14 +430,13 @@ public class TestChunkCoding {
 
     @Test
     public void testChunkedOutputStreamSmallChunk() throws IOException {
-        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
-        ChunkedOutputStream out = new ChunkedOutputStream(
-                new SessionOutputBufferMock(buffer), 2);
+        final ByteArrayOutputStream buffer = new ByteArrayOutputStream();
+        final ChunkedOutputStream out = new ChunkedOutputStream(2, new SessionOutputBufferMock(buffer));
         out.write('1');
         out.finish();
         out.close();
 
-        byte [] rawdata =  buffer.toByteArray();
+        final byte [] rawdata =  buffer.toByteArray();
 
         Assert.assertEquals(11, rawdata.length);
         Assert.assertEquals('1', rawdata[0]);
@@ -439,12 +454,12 @@ public class TestChunkCoding {
 
     @Test
     public void testResumeOnSocketTimeoutInData() throws IOException {
-        String s = "5\r\n01234\r\n5\r\n5\0006789\r\na\r\n0123\000456789\r\n0\r\n";
-        SessionInputBuffer sessbuf = new SessionInputBufferMock(
+        final String s = "5\r\n01234\r\n5\r\n5\0006789\r\na\r\n0123\000456789\r\n0\r\n";
+        final SessionInputBuffer sessbuf = new SessionInputBufferMock(
                 new TimeoutByteArrayInputStream(s.getBytes("ISO-8859-1")), 16);
-        InputStream in = new ChunkedInputStream(sessbuf);
+        final InputStream in = new ChunkedInputStream(sessbuf);
 
-        byte[] tmp = new byte[3];
+        final byte[] tmp = new byte[3];
 
         int bytesRead = 0;
         int timeouts = 0;
@@ -456,22 +471,23 @@ public class TestChunkCoding {
                 if (i > 0) {
                     bytesRead += i;
                 }
-            } catch (InterruptedIOException ex) {
+            } catch (final InterruptedIOException ex) {
                 timeouts++;
             }
         }
         Assert.assertEquals(20, bytesRead);
         Assert.assertEquals(2, timeouts);
-    }
+        in.close();
+}
 
     @Test
     public void testResumeOnSocketTimeoutInChunk() throws IOException {
-        String s = "5\000\r\000\n\00001234\r\n\0005\r\n56789\r\na\r\n0123456789\r\n\0000\r\n";
-        SessionInputBuffer sessbuf = new SessionInputBufferMock(
+        final String s = "5\000\r\000\n\00001234\r\n\0005\r\n56789\r\na\r\n0123456789\r\n\0000\r\n";
+        final SessionInputBuffer sessbuf = new SessionInputBufferMock(
                 new TimeoutByteArrayInputStream(s.getBytes("ISO-8859-1")), 16);
-        InputStream in = new ChunkedInputStream(sessbuf);
+        final InputStream in = new ChunkedInputStream(sessbuf);
 
-        byte[] tmp = new byte[3];
+        final byte[] tmp = new byte[3];
 
         int bytesRead = 0;
         int timeouts = 0;
@@ -483,13 +499,14 @@ public class TestChunkCoding {
                 if (i > 0) {
                     bytesRead += i;
                 }
-            } catch (InterruptedIOException ex) {
+            } catch (final InterruptedIOException ex) {
                 timeouts++;
             }
         }
         Assert.assertEquals(20, bytesRead);
         Assert.assertEquals(5, timeouts);
-    }
+        in.close();
+}
 
 }
 
