@@ -24,117 +24,114 @@
  * <http://www.apache.org/>.
  *
  */
-
 package org.apache.http.message;
 
 import java.util.NoSuchElementException;
 
-import junit.framework.TestCase;
-
 import org.apache.http.Header;
 import org.apache.http.HeaderElement;
 import org.apache.http.HeaderElementIterator;
+import org.junit.Assert;
+import org.junit.Test;
 
 /**
  * Tests for {@link BasicHeaderElementIterator}.
  *
  */
-public class TestBasicHeaderElementIterator extends TestCase {
-
-    // ------------------------------------------------------------ Constructor
-    public TestBasicHeaderElementIterator(String testName) {
-        super(testName);
-    }
+public class TestBasicHeaderElementIterator {
 
+    @Test
     public void testMultiHeader() {
-        Header[] headers = new Header[]{
+        final Header[] headers = new Header[]{
                 new BasicHeader("Name", "value0"),
                 new BasicHeader("Name", "value1")
         };
 
-        HeaderElementIterator hei =
+        final HeaderElementIterator hei =
                 new BasicHeaderElementIterator(
                         new BasicHeaderIterator(headers, "Name"));
 
-        assertTrue(hei.hasNext());
-        HeaderElement elem = (HeaderElement) hei.next();
-        assertEquals("The two header values must be equal",
+        Assert.assertTrue(hei.hasNext());
+        HeaderElement elem = hei.nextElement();
+        Assert.assertEquals("The two header values must be equal",
                 "value0", elem.getName());
 
-        assertTrue(hei.hasNext());
-        elem = (HeaderElement)hei.next();
-        assertEquals("The two header values must be equal",
+        Assert.assertTrue(hei.hasNext());
+        elem = hei.nextElement();
+        Assert.assertEquals("The two header values must be equal",
                 "value1", elem.getName());
 
-        assertFalse(hei.hasNext());
+        Assert.assertFalse(hei.hasNext());
         try {
             hei.next();
-            fail("NoSuchElementException should have been thrown");
-        } catch (NoSuchElementException ex) {
+            Assert.fail("NoSuchElementException should have been thrown");
+        } catch (final NoSuchElementException ex) {
             // expected
         }
 
-        assertFalse(hei.hasNext());
+        Assert.assertFalse(hei.hasNext());
         try {
             hei.next();
-            fail("NoSuchElementException should have been thrown");
-        } catch (NoSuchElementException ex) {
+            Assert.fail("NoSuchElementException should have been thrown");
+        } catch (final NoSuchElementException ex) {
             // expected
         }
     }
 
+    @Test
     public void testMultiHeaderSameLine() {
-        Header[] headers = new Header[]{
+        final Header[] headers = new Header[]{
                 new BasicHeader("name", "value0,value1"),
                 new BasicHeader("nAme", "cookie1=1,cookie2=2")
         };
 
-        HeaderElementIterator hei =
+        final HeaderElementIterator hei =
                 new BasicHeaderElementIterator(new BasicHeaderIterator(headers, "Name"));
 
-        HeaderElement elem = (HeaderElement)hei.next();
-        assertEquals("The two header values must be equal",
+        HeaderElement elem = hei.nextElement();
+        Assert.assertEquals("The two header values must be equal",
                 "value0", elem.getName());
-        elem = (HeaderElement)hei.next();
-        assertEquals("The two header values must be equal",
+        elem = hei.nextElement();
+        Assert.assertEquals("The two header values must be equal",
                 "value1", elem.getName());
-        elem = (HeaderElement)hei.next();
-        assertEquals("The two header values must be equal",
+        elem = hei.nextElement();
+        Assert.assertEquals("The two header values must be equal",
                 "cookie1", elem.getName());
-        assertEquals("The two header values must be equal",
+        Assert.assertEquals("The two header values must be equal",
                 "1", elem.getValue());
 
-        elem = (HeaderElement)hei.next();
-        assertEquals("The two header values must be equal",
+        elem = hei.nextElement();
+        Assert.assertEquals("The two header values must be equal",
                 "cookie2", elem.getName());
-        assertEquals("The two header values must be equal",
+        Assert.assertEquals("The two header values must be equal",
                 "2", elem.getValue());
     }
 
+    @Test
     public void testFringeCases() {
-        Header[] headers = new Header[]{
+        final Header[] headers = new Header[]{
                 new BasicHeader("Name", null),
                 new BasicHeader("Name", "    "),
                 new BasicHeader("Name", ",,,")
         };
 
-        HeaderElementIterator hei =
+        final HeaderElementIterator hei =
                 new BasicHeaderElementIterator(
                         new BasicHeaderIterator(headers, "Name"));
 
-        assertFalse(hei.hasNext());
+        Assert.assertFalse(hei.hasNext());
         try {
             hei.next();
-            fail("NoSuchElementException should have been thrown");
-        } catch (NoSuchElementException ex) {
+            Assert.fail("NoSuchElementException should have been thrown");
+        } catch (final NoSuchElementException ex) {
             // expected
         }
 
-        assertFalse(hei.hasNext());
+        Assert.assertFalse(hei.hasNext());
         try {
             hei.next();
-            fail("NoSuchElementException should have been thrown");
-        } catch (NoSuchElementException ex) {
+            Assert.fail("NoSuchElementException should have been thrown");
+        } catch (final NoSuchElementException ex) {
             // expected
         }
     }
