@@ -32,68 +32,63 @@ import java.io.ByteArrayOutputStream;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 
-import junit.framework.TestCase;
-
 import org.apache.http.HttpVersion;
 import org.apache.http.RequestLine;
+import org.junit.Assert;
+import org.junit.Test;
 
 /**
  * Simple tests for {@link RequestLine}.
- *
- *
  */
-public class TestRequestLine extends TestCase {
-
-    // ------------------------------------------------------------ Constructor
-    public TestRequestLine(String testName) {
-        super(testName);
-    }
-
-    // ------------------------------------------------------- TestCase Methods
+public class TestRequestLine {
 
+    @Test
     public void testConstructor() {
-        RequestLine requestline = new BasicRequestLine("GET", "/stuff", HttpVersion.HTTP_1_1);
-        assertEquals("GET", requestline.getMethod());
-        assertEquals("/stuff", requestline.getUri());
-        assertEquals(HttpVersion.HTTP_1_1, requestline.getProtocolVersion());
+        final RequestLine requestline = new BasicRequestLine("GET", "/stuff", HttpVersion.HTTP_1_1);
+        Assert.assertEquals("GET", requestline.getMethod());
+        Assert.assertEquals("/stuff", requestline.getUri());
+        Assert.assertEquals(HttpVersion.HTTP_1_1, requestline.getProtocolVersion());
     }
 
+    @Test
     public void testConstructorInvalidInput() {
         try {
             new BasicRequestLine(null, "/stuff", HttpVersion.HTTP_1_1);
-            fail("IllegalArgumentException should have been thrown");
-        } catch (IllegalArgumentException e) { /* expected */ }
+            Assert.fail("IllegalArgumentException should have been thrown");
+        } catch (final IllegalArgumentException e) { /* expected */ }
         try {
             new BasicRequestLine("GET", null, HttpVersion.HTTP_1_1);
-            fail("IllegalArgumentException should have been thrown");
-        } catch (IllegalArgumentException e) { /* expected */ }
+            Assert.fail("IllegalArgumentException should have been thrown");
+        } catch (final IllegalArgumentException e) { /* expected */ }
         try {
             new BasicRequestLine("GET", "/stuff", (HttpVersion)null);
-            fail("IllegalArgumentException should have been thrown");
-        } catch (IllegalArgumentException e) { /* expected */ }
+            Assert.fail("IllegalArgumentException should have been thrown");
+        } catch (final IllegalArgumentException e) { /* expected */ }
     }
 
+    @Test
     public void testCloning() throws Exception {
-        BasicRequestLine orig = new BasicRequestLine("GET", "/stuff", HttpVersion.HTTP_1_1);
-        BasicRequestLine clone = (BasicRequestLine) orig.clone();
-        assertEquals(orig.getMethod(), clone.getMethod());
-        assertEquals(orig.getUri(), clone.getUri());
-        assertEquals(orig.getProtocolVersion(), clone.getProtocolVersion());
+        final BasicRequestLine orig = new BasicRequestLine("GET", "/stuff", HttpVersion.HTTP_1_1);
+        final BasicRequestLine clone = (BasicRequestLine) orig.clone();
+        Assert.assertEquals(orig.getMethod(), clone.getMethod());
+        Assert.assertEquals(orig.getUri(), clone.getUri());
+        Assert.assertEquals(orig.getProtocolVersion(), clone.getProtocolVersion());
     }
 
+    @Test
     public void testSerialization() throws Exception {
-        BasicRequestLine orig = new BasicRequestLine("GET", "/stuff", HttpVersion.HTTP_1_1);
-        ByteArrayOutputStream outbuffer = new ByteArrayOutputStream();
-        ObjectOutputStream outstream = new ObjectOutputStream(outbuffer);
+        final BasicRequestLine orig = new BasicRequestLine("GET", "/stuff", HttpVersion.HTTP_1_1);
+        final ByteArrayOutputStream outbuffer = new ByteArrayOutputStream();
+        final ObjectOutputStream outstream = new ObjectOutputStream(outbuffer);
         outstream.writeObject(orig);
         outstream.close();
-        byte[] raw = outbuffer.toByteArray();
-        ByteArrayInputStream inbuffer = new ByteArrayInputStream(raw);
-        ObjectInputStream instream = new ObjectInputStream(inbuffer);
-        BasicRequestLine clone = (BasicRequestLine) instream.readObject();
-        assertEquals(orig.getMethod(), clone.getMethod());
-        assertEquals(orig.getUri(), clone.getUri());
-        assertEquals(orig.getProtocolVersion(), clone.getProtocolVersion());
+        final byte[] raw = outbuffer.toByteArray();
+        final ByteArrayInputStream inbuffer = new ByteArrayInputStream(raw);
+        final ObjectInputStream instream = new ObjectInputStream(inbuffer);
+        final BasicRequestLine clone = (BasicRequestLine) instream.readObject();
+        Assert.assertEquals(orig.getMethod(), clone.getMethod());
+        Assert.assertEquals(orig.getUri(), clone.getUri());
+        Assert.assertEquals(orig.getProtocolVersion(), clone.getProtocolVersion());
     }
 
 }
